/***************************************************************************
 *   Copyright (C) 2005, 2006 by Hannes Hauswedell                         *
 *   hannes.hauswedell@gmail.com                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __BSDPORT_H__
#define __BSDPORT_H__


#include <iostream>
#include <string>
#include <stdlib.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/errno.h>

#include <qstring.h>
#include <qstringlist.h>
#include <qlistview.h>
#include <qfile.h>
#include <qdir.h>
#include <qprocess.h>
#include <qobject.h>
#include <qapplication.h>
#include <qsignalmapper.h>
#include <qmutex.h>
#include <qevent.h>

#include <kportsview.h>
#include <kprocio.h>

//#include <sys/errno.h>

//using namespace std;

class T_function;
class QMyEvent;
class QBsdPortGlobal;	
class QBsdPort;
class QBsdPortPkg;
class QBsdPortInstPkg;
class QBsdPortAvSrcPkg;
class QBsdPortAvBinPkg;

class KProcess;
class KMyProc;


/** concerningg structure of this file and classes:

	QBsdPort global stores all data that is static and global to the other
	classes. it is necessary because i want signals and slots for the
	global functions which would not be available if the were static members
	of QBsdPort.

	QBsdPort is the main object and stores all data concerning one port,
	i.e. PORT REFERS TO A PIECE OF SOFTWARE HAVING A UNIQUE LOCATION 
	(ORIGIN) IN THE FREEBSD PORTSTREE.

	a QBsdPort object has three possible sub-objects, one each of 
	QBsdPortInstPkg, QBsdPortAvSrcPkg + QBsdPortAvBinPkg - who all
	inherit from QBsdPortPkg (who is never used directly).

	QBsdPortInstPkg stores all relevant data of the installed package
	 of the port if one is installed.

	QBsdPortAvSrcPkg represents what is usually referred to as "port",
	 it stores all data available for the source install.

	QBsdPortAvBinPkg is the same for the binary tarball available on ftp

	T_function is a class that is used to create dynamic thread objects,
	a fucntion pointer is passed to the constructor, when thread is executed
	the funciton is run. to put it simple, you can create a thread to every
	void-void function. the thread emits a signal upon finishing and deletes itself



	all the bogus with custom events will dissappear, as soon
	as kde switches to qt4.0 (threads are not qobjects in qt<4.0)
**/


class T_function : public QThread  /// this class is used to dynamically create threaded functions
{
//Q_OBJECT
private:
	void (QBsdPortGlobal::*function1)(void);
	void (QBsdPortGlobal::*function1a)(QString Term, int Criterium, bool Exact);
	void (QBsdPortGlobal::*function1b)(int build);
	void (QBsdPortGlobal::*function1c)(QBsdPort*, int, int, int, bool, QString);	/// for install
	void (QBsdPortGlobal::*function1d)(QBsdPort*, int, bool); ///and uninstall
//	QBsdPortGlobal *localGlobal;
	
	void (QBsdPort::*function2)(void);

	void (QBsdPortInstPkg::*function3)(void);
	void (QBsdPortInstPkg::*function3b)(QString dest_file);

	void (QBsdPortAvSrcPkg::*function4)(void);

	void (QBsdPortAvBinPkg::*function5)(void);

	void *parent;

	int whichParent;		/// used to identify what kind of parent is used
	int ident;			/// used to identify the passed fucntion so the emitSignal funciton in parent may be called
	///the following are used for different things
	
	QBsdPort *bsdport;
	QString	string;
	int	integer[3];
	bool	boolean;
	
	
public:
	/// not implemented inline, because it needs access to static vars	
	T_function(void (QBsdPortGlobal::*Function)(void), int Ident);
	T_function(void (QBsdPortGlobal::*Function)(QString, int, bool), int Ident, QString Term, int Criterium, bool Exact);
	T_function(void (QBsdPortGlobal::*Function)(int), int Ident, int build);
	T_function(void (QBsdPortGlobal::*Function)(QBsdPort*, int, int, int, bool, QString), int Ident,
		QBsdPort* Port, int aim, int recursive, int binsource, bool ignore, QString env);
	T_function(void (QBsdPortGlobal::*Function)(QBsdPort*, int, bool), int Ident,
		QBsdPort* Port, int recursive, bool ignore);	
	
	T_function(QBsdPort *Parent, void (QBsdPort::*Function)(void), int Ident)
	{
		parent 		= (void*)Parent;
//		parent2		= Parent;
		function2 	= Function;
		ident 		= Ident;
		whichParent 	= 2;
	}
	T_function(QBsdPortInstPkg *Parent, void (QBsdPortInstPkg::*Function)(void), int Ident)
	{
		parent 		= (void*)Parent;
//		parent3		= Parent;
		function3 	= Function;
		ident 		= Ident;
		whichParent 	= 3;
	}
	T_function(QBsdPortInstPkg *Parent, void (QBsdPortInstPkg::*Function)(QString), int Ident, QString dest_file) /// only for exportpkg
	{
		parent 		= (void*)Parent;
//		parent3		= Parent;
		function3b 	= Function;
		ident 		= Ident;
		whichParent 	= 3;
		string = dest_file;
	}
	T_function(QBsdPortAvSrcPkg *Parent, void (QBsdPortAvSrcPkg::*Function)(void), int Ident)
	{
		parent 		= (void*)Parent;
//		parent4		= Parent;
		function4 	= Function;
		ident 		= Ident;
		whichParent 	= 4;
	}
	T_function(QBsdPortAvBinPkg *Parent, void (QBsdPortAvBinPkg::*Function)(void), int Ident)
	{
		parent 		= (void*)Parent;
//		parent4		= Parent;
		function5	= Function;
		ident 		= Ident;
		whichParent 	= 5;
	}

	virtual void run();

/*signals:
	void finished();*/
};

class QMyEvent : public QObject, public QCustomEvent
{
Q_OBJECT

public:
	QString s;
	QPtrList <QBsdPort> l;
	int	i;
	
	QMyEvent(int type, QString string) 
		: QCustomEvent(type), s(string), QObject() {}
		
	QMyEvent(int type, int integer) 
		: QCustomEvent(type), QObject() { i = integer; }
		
	QMyEvent(int type, QPtrList <QBsdPort> list) 
		: QCustomEvent(type), l(list), QObject(){ }
};


class QBsdPortGlobal : public QObject
{
Q_OBJECT
public:
	/// constructor doesnt need to do anything special
	QBsdPortGlobal() { array.setAutoDelete(true); }


	QPtrList <QBsdPort> array;	/// this array stores all our pkgs

	/** environment stuff **/
	void	guessPlatform();	/// tries to get members from
	void	guessOtherEnv();
	QString	portsDir,		/// portsdir, defaults to /usr/ports
		indexPath,		/// index file, defaults to /usr/ports/INDEX-5
		pkgSite,		/// ftp location with packages
		portsDbDir,		/// ports db, defaults to /var/db/ports
		pkgDbDir;		/// pkg db, defaults to /var/db/pkg
	QString	multiPorts;		/// software that has multiple ports, with equal name

	QMutex	getMutex;
						
	struct pl_struct
	{
		int pkgsystem;		/**    -1 => freebsd ports without portupgrade (deprecated)
						0 => freebsd ports
						1 => openbsd ports
						2 => pkgsrc
						3 => mirports
						4 => t2-ports
						5 => crux-ports
						6 => ... whatever **/
		int os;			/** depends on pkgsystem pick and is useless most of the time
						but important for example for platform = 2, because
						syntax is different on non-netbsd platforms **/
		QString release;	/// a string representing the release, e.g. "6-stable"
		QString mirror;		/// two letter string that represents the country
	};
	
	pl_struct platform;
		
	int	pkgCount;		/// total amount of pkgs
	bool	portaudit,		/// whether portaudit is available
		portsnatch;		/// whether portsnatch is available
	int	debug;			/// activate output to console for debugging
	ulong	updDate;		/// minimum date for UPDATING file warnings
				

	bool	keepLogsForSuccessfulProcs; /// what it says
	bool	combinePkgInfo;		/// if a certain pkg e.g. avBin doesnt provide info for e.g. files get it from avsrc
	
	bool	firstRun;
	
		
	/** these function build retrieve information for KPORTS and dont modify
	    any on-disk data. they should be run in order from top to bottom **/
	    
	/// add all ports to array ( INDEX file is read )
	void	findAllAvailableSrc();
	/// add installed packages to array (read output from "pkg_info -apcdDrRLso")
	void	findAllInstalled();
	/// for now only get available versions from ftp (function for building ftp INDEX file needed)
	void	findAllAvailableBin();
	/// read the updating file and add warnings if necessary (only installed versions)
	void	readUpdFile();
	/// do a portaudit check on all installed pkgs (available versions are checked seperately)
	void	findVulnas();
	
	
	/// search the array ports fulfillining search criteria
	QPtrList <QBsdPort>	search(QString Term, int Criterium, bool Exact);
	void	searchFrame(QString Term, int Criterium, bool Exact);		/// aframe around the above whihc returns void and signals out the ptrlist

	/// return port with corresponding origin
	QBsdPort	*portWithOrigin(QString Origin);
	/// return port with corresponding name (and other vars
	QBsdPort 	*portWithName(QString Name, int Package, QString Version);



	/** these functions update various on-disk databases and the like. they
	    do NOT upGRADE anything **/
	
	/// update portaudit's database
	void	updateVulnaDb();
	
	/// update the ports directory using cvs and the settings from preferences
	void	updatePortsDir();
	/// rebuild or refetch the INDEX file
	void	updateIndex(int build);
	/// update the portsDb (only for pkgsystem == 0)
	void	updatePortsDb();





	/// install or uninstall a port
	void	install_port(QBsdPort*, int Aim, int Recursive, int BinSource, bool Ignore, QString Env);
	/** returns the command string to install this port with the given options (this needs to run by run_cmd in global)	
		aims: 	0: fresh install
			1: upgrade
			2: downgrade
			3: reinstall
		recursive:
			0: do not act on ports depending on this one
			1: recursive (requirements)
			2: upward recusrsive (dependencies)
			3: recursive and upward recursive
		binsource
			0: only use ports
			1: only use pkgs
			2: prefer pkgs but use ports if they are newer or no pkg exists
		ignore:(keep going option in portupgrade)
			0: on failures or errors abort
			1: keep going on failures or errors
		
	**/

	void	uninstall_port(QBsdPort*, int Recursive, bool Ignore);
	/** returns the command string to uninstall this port with the given options (this needs to be run by run_cmd in global)
		recur:	0: do not act on ports depending on this one
			1: recursive 
			2: upward recusrsive (reverse dependecies)
			3: recursive and upward recursive
		ignore:(keep going option in portupgrade)
			0: on failures or errors abort
			1: keep going on failures or errors	
	**/


/** ----- threads to all the other functions ------- **/


	void	T_findAllAvailableSrc();
	void	T_findAllInstalled();
	void	T_findAllAvailableBin();
	void	T_readUpdFile();
	void	T_findVulnas();
	void	T_search(QString Term, int Criterium, bool Exact);
	
	void	T_updateVulnaDb();
	void	T_updatePortsDir();
	void	T_updateIndex(int build);
	void	T_updatePortsDb();
	
	void	T_install_port(QBsdPort* Port, int Aim, int Recursive, int BinSource, bool Ignore, QString Env);
	void	T_uninstall_port(QBsdPort* Port, int Recursive, bool Ignore);






	void customEvent(QCustomEvent *ee)
	{
		QMyEvent* e = (QMyEvent*)ee;
		
		if (e->type()<1000)	/// not intended for this
			return;
		
		int signal = e->type() - 1000;

		
		switch(signal)
		{
//			case -10: emit vulnaDbOutdated(); break;
//			cas -9: emit vulnaDbUpdateUnsuccessful(); break;
			case 0: emit T_findAllAvailableSrcFinished(); break;
			case 1: emit T_findAllInstalledFinished(); break;
			case 2: emit T_findAllAvailableBinFinished(); break;
			case 3: emit T_readUpdFileFinished(); break;
			case 4: emit T_findVulnasFinished(); break;
			case 5: emit T_updateVulnaDbFinished(); break;

			case 108:{
//				int *ptr = (int*)e->data();
				emit findAllAvBinProgress(e->i);
//				if (*ptr>=3)
//					delete ptr; /// is not needed any longer, since value not reference is passed
				} break;
			case 110: emit indexFileNotFound(); break;
			case 111: emit vulnaDbOutdated(); break;
			case 112: emit vulnaDbUpdateSuccessful(); break;
			case 113:{ 
//				QString *ptr = (QString*)e->data();
				emit vulnaDbUpdateUnsuccessful(e->s);// delete ptr; /// is not needed any longer
				} break;
			case 120:{
//				QPtrList <QBsdPort> *liste = (QPtrList <QBsdPort> *)e->data();
				emit searchFinished(e->l); //delete e->data(); /// is not needed any longer
				} break;
			/// updatePorts signals
			case 140:{
//				int *ptr = (int*)e->data();
				emit percentChanged(e->i); /// not deleted int is pointer still used by function
				}break;
			case 141:{
//				QString *ptr = (QString*)e->data();
				emit error(e->s); //delete ptr; /// is not needed any longer
				}break;
					
			case 199:{
//				QString *ptr = (QString*)e->data();
				emit installCmd(e->s);// delete ptr; /// is not needed any longer
				}break;
			case 200:{
//				QString *ptr = (QString*)e->data();
				emit installStdOut(e->s); //delete ptr; /// is not needed any longer
				}break;
			case 201:{
//				QString *ptr = (QString*)e->data();
				emit installErrOut(e->s); //delete ptr; /// is not needed any longer
				}break;
			case 202: emit installFinished(); break;
			
		}



	}

private:
			
	QStringList activeProcs; 	/// holds the names of currently active QProcesses
	QStringList toBeStoppedProcs;	/// holds the names of active Processes that should be interrupted


	int	updateFTPIndex();	/// returns: 1 = needs no updating or updated succesfully, 0 = error
	
	uint	ui_user_id;
	
	bool	becomeRoot();
	bool	becomeUser();

signals:
/**normal signals**/
	void	indexFileNotFound();
	void	vulnaDbOutdated();
	void	vulnaDbUpdateSuccessful();
	void	vulnaDbUpdateUnsuccessful(QString);
	void	searchFinished(QPtrList <QBsdPort>);
	void	percentChanged(int);
	void	error(QString);
	void	findAllAvBinProgress(int);
	/**	0 = deleting old data
		1 = fetching new information from server
		2 = information fetched successfully and reading it now
		3 = error during update
		4 = finished succesfully
	**/
	
	void	installCmd(QString);
	void	installStdOut(QString);
	void	installErrOut(QString);
	void	installFinished();
	

/**thread-finished signals **/
	void	T_findAllAvailableSrcFinished();
	void	T_findAllInstalledFinished();
	void	T_findAllAvailableBinFinished();
	void	T_readUpdFileFinished();
	void	T_findVulnasFinished();
	void	T_updateVulnaDbFinished();
	
public slots:
			
	void	stopProc(QString Name);
	
private slots:
	void	postAllEvents();
};


class QBsdPort : public QObject
{

Q_OBJECT
private:
	QString m_origin;
	QBsdPortInstPkg	*m_installed;
	QBsdPortAvSrcPkg *m_avSource;
	QBsdPortAvBinPkg *m_avBinary;

public:

	static QBsdPortGlobal *global;

/** -------- code for indivuidual objects ----------- **/
	QBsdPort() 		: m_origin ( 0 )	{ init(); }
	QBsdPort(QString origin): m_origin( origin )	{ init(); }
	~QBsdPort()					{ global->pkgCount--; }

	void init()
	{
		global->pkgCount++;
		m_installed = NULL;
		m_avSource = NULL;
		m_avBinary = NULL;
	}
	
	/** find calls **/
	void	findInstalled();
	void	findAvSource();
	void	findAvBinary();

	/** get calls **/
	QBsdPortInstPkg 	*installed()	const { return m_installed;}
	QBsdPortAvSrcPkg	*avSource() 	const { return m_avSource; }
	QBsdPortAvBinPkg	*avBinary()  	const { return m_avBinary; }
	QString	origin() const			{ return m_origin; }

	/** set calls **/
	void	setOrigin(QString newValue)		{ m_origin = newValue; }
	void	setInstalled(QBsdPortInstPkg *newValue);
	void	setAvSource(QBsdPortAvSrcPkg *newValue);
	void	setAvBinary(QBsdPortAvBinPkg *newValue);



/**----- threaded calls for all functions, for descriptionn look above -------**/
	void	T_findInstalled();
	void	T_findAvBinary();
	void	T_findAvSource();


	void customEvent(QCustomEvent *e)
	{
		if (e->type()<1000)	/// not intended for this
			return;
		
		int signal = e->type() - 1000;

		switch(signal)
		{
			case 0: emit T_findInstalledFinished(); break;
			case 1: emit T_findAvSourceFinished(); break;
			case 2: emit T_findAvBinaryFinished(); break;
		}


	}

signals:
	void	T_findInstalledFinished();
	void	T_findAvSourceFinished();
	void	T_findAvBinaryFinished();

};


class QBsdPortPkg : public QObject /// pure virtual
{
Q_OBJECT
private:
	QString m_version,	/// version
		m_name,		/// name; normally the same, but some ports change name on options 
		m_files,	/// contained/installed files
		m_reqs,		/// requirements
		m_vulna,	/// known vulnarabilities (stored as "description|reference")
		m_prefix,	/// prefix this package was/will be installed to
		m_comment,	/// one-line comment
		m_www,		/// webpage
		m_descr,	/// long description
		m_categories,	/// categories for this port
		m_maint;	/// port-maintainers email-address
	ulong	m_size;		/// size occupied by files in package

	QBsdPort *m_parent;	/// pointer to parent

	/** if a member of instpkg or avbinpkg equals that of avsrcpkg, it is not saved but instead
		an "s" is saved in the string. this should save alot of memory. the get calls
		translate this beahviour so that no extra code is neeed anywhere else.
		ATTENTION: members of QBSdPortAvSrcPkg MUST NEVER be "s" or get calls will
		loop infintely!!!!! **/

public:

	QBsdPortPkg() : m_version( 0 ), m_vulna("no_init") {}
	QBsdPortPkg(QBsdPort* Parent, QString Version) : m_version(Version), m_vulna("no_init") { m_parent = Parent;}

	/** some calls are reimplemented by the children **/
	void findVulna();		/// information available with portaudit (freebsd ports only)

	/** get calls, some arent inline cause they need access to other class**/
	QString version() const;
	QString name() const;
	QString files() const;
	QString reqs() const;
	QString vulna() const;
	QString prefix() const;
	QString comment() const;
	QString descr() const;
	QString www() const;
	QString categories()	const { return m_categories; }
	QString maint() 	const { return m_maint; }
	ulong	size() const	{ return m_size; }
	QBsdPort *parent() const { return m_parent; }

	/** set calls **/
	void	setVersion(QString newValue)	{ m_version = newValue; }
	void	setName(QString newValue)	{ m_name = newValue; }
	void	setFiles(QString newValue)	{ m_files = newValue; }
	void	setReqs(QString newValue)	{ m_reqs = newValue; }
	void	setVulna(QString newValue)	{ m_vulna = newValue; }
	void	setPrefix(QString newValue)	{ m_prefix = newValue; }
	void	setComment(QString newValue)	{ m_comment = newValue; }
	void	setDescr(QString newValue)	{ m_descr = newValue; }
	void	setWWW(QString newValue)	{ m_www = newValue; }
	void	setSize(ulong newValue)		{ m_size = newValue; }
	void	setParent(QBsdPort* newValue)	{ m_parent = newValue; }
	void	setCategories(QString newValue)	{ m_categories = newValue; }
	void	setMaint(QString newValue)	{ m_maint = newValue; }



};


class QBsdPortInstPkg : public QBsdPortPkg
{
Q_OBJECT
private:
	/** additional members for installed versions **/
	QString	m_deps,		/// dependencies
		m_message,	/// post installation message
		m_updWarning,	/// warnings from the updating file
//		m_options,	/// options this package was isntalled with
		m_falseFiles;	/// files with mismatching md5s, an ! after the filepath marks missing files
		
	QMutex	mutex[4];

public:
	QBsdPortInstPkg() : QBsdPortPkg() {}
	QBsdPortInstPkg(QBsdPort* Parent, QString Version) : QBsdPortPkg(Parent, Version) {}

	/** additional find calls **/
	void	findFalseFiles();
	void	findSize();

	void exportPkg(QString dest_file);	/// create a file for this package

	/** the above, as threads **/
	void	T_findVulna()
	{
		if (mutex[0].tryLock())
		{
			T_function *thread = new T_function(this, &QBsdPortInstPkg::findVulna, 10);
			thread->start();
		}
	}
	void	T_findFalseFiles()
	{
		if (mutex[1].tryLock())
		{
			T_function *thread = new T_function(this, &QBsdPortInstPkg::findFalseFiles, 0);
			thread->start();
		}
	}
	void	T_findSize()
	{
		if (mutex[2].tryLock())
		{
			T_function *thread = new T_function(this, &QBsdPortInstPkg::findSize, 1);
			thread->start();
		}
	}
	void	T_exportPkg(QString dest_file)
	{
		if (mutex[3].tryLock())
		{
			T_function *thread = new T_function(this, &QBsdPortInstPkg::exportPkg, 2, dest_file);
			thread->start();
		}
	}


	/** additional get calls for new members **/
	QString deps() const		{ return m_deps; }
	QString message() const		{ return m_message; }
	QString updWarning() const 	{ return m_updWarning; }
//	QString options() const 	{ return m_options; }
	QString falseFiles() const  	{ return m_falseFiles; }



	/** additional set calls for new members **/
	void	setDeps(QString newValue)	{ m_deps = newValue; }
	void	setMessage(QString newValue)	{ m_message = newValue; }
	void	setUpdWarning(QString newValue)	{ m_updWarning = newValue; }
//	void	setOptions(QString newValue)	{ m_options = newValue; }
	void	setFalseFiles(QString newValue) { m_falseFiles = newValue; }

	void customEvent(QCustomEvent *e)
	{
		if (e->type()<1000)	/// not intended for this
			return;
		
		int signal = e->type() - 1000;

		switch(signal)
		{
			case 10: mutex[0].unlock(); emit T_findVulnaFinished(); break;
			case 0: mutex[1].unlock(); emit T_findFalseFilesFinished(); break;
			case 1: mutex[2].unlock(); emit T_findSizeFinished(); break;
			case 2: mutex[3].unlock(); emit T_exportPkgFinished(); break;
			
			case 100: emit exportPkgSuccessful(); break;
			case 101:
				QString *ptr = (QString*)e->data();
				emit exportPkgUnsuccessful(*ptr); break;
		}


	}

signals:
	void	T_findVulnaFinished();
	void	T_findFalseFilesFinished();
	void	T_findSizeFinished();
	void	T_exportPkgFinished();
	
	void	exportPkgSuccessful();
	void	exportPkgUnsuccessful(QString);

};

class QBsdPortAvBinPkg : public QBsdPortPkg //right now avbin doesnt differ from parent
{
Q_OBJECT
private:
	/** needs no additional members **/

	QMutex	mutex[1];
public:
	QBsdPortAvBinPkg() : QBsdPortPkg() {}
	QBsdPortAvBinPkg(QBsdPort* Parent, QString Version) : QBsdPortPkg(Parent, Version) {}

	void	T_findVulna()
	{
		if (mutex[0].tryLock())
		{
			T_function *thread = new T_function(this, &QBsdPortAvBinPkg::findVulna, 10);
			thread->start();
		}
	}

	/** additional get calls for new members **/



	/** additional set calls for new members **/
	void customEvent(QCustomEvent *e)
	{
		if (e->type()<1000)	/// not intended for this
			return;
		
		int signal = e->type() - 1000;

		switch(signal)
		{
			case 10: mutex[0].unlock(); emit T_findVulnaFinished(); break;
		}
	}
signals:
	void	T_findVulnaFinished();
};

class QBsdPortAvSrcPkg : public QBsdPortPkg
{
Q_OBJECT
private:
	/** additional members for installed versions **/
	QString	/// certain flags for ports
		m_broken,
		m_forbidden,
		m_ignore,
		m_deprecated,
		m_restricted;
	QString m_options;	/// build options
	
	QMutex	mutex[6];

public:
	QBsdPortAvSrcPkg() : QBsdPortPkg() {}
	QBsdPortAvSrcPkg(QBsdPort* Parent, QString Version) : QBsdPortPkg(Parent, Version) {}

	/** additional find calls **/
	void	findOptions();		
	void	findFiles();		/// files are not found with parent find call
	void	findDescr();		/// neither description
	void	findFlags();		/// find flags like igonre or broken

	void exportPkg();	/// create a file for this package

	/** the above, as threads **/
	void	T_findVulna()
	{
		if (mutex[0].tryLock())
		{
			T_function *thread = new T_function(this, &QBsdPortAvSrcPkg::findVulna, 10);
			thread->start();
		}
	}
	void	T_findOptions()
	{
		if (mutex[1].tryLock())
		{
			T_function *thread = new T_function(this, &QBsdPortAvSrcPkg::findOptions, 0);
			thread->start();
		}
	}
	void	T_findFiles()
	{
		if (mutex[2].tryLock())
		{
			T_function *thread = new T_function(this, &QBsdPortAvSrcPkg::findFiles, 1);
			thread->start();
		}
	}
	void	T_findDescr()
	{
		if (mutex[3].tryLock())
		{
			T_function *thread = new T_function(this, &QBsdPortAvSrcPkg::findDescr, 2);
			thread->start();
		}
	}
	void	T_exportPkg()
	{
		if (mutex[4].tryLock())
		{
			T_function *thread = new T_function(this, &QBsdPortAvSrcPkg::exportPkg, 3);
			thread->start();
		}
	}
	void	T_findFlags()
	{
		if (mutex[5].tryLock())
		{
			T_function *thread = new T_function(this, &QBsdPortAvSrcPkg::findFlags, 4);
			thread->start();
		}
	}

	/** additional get calls for new members **/
	QString options() 	const { return m_options; }
	QString forbidden() 	const { return m_forbidden; }
	QString ignore() 	const { return m_ignore; }
	QString restricted() 	const { return m_restricted; }
	QString broken() 	const { return m_broken; }
	QString deprecated() 	const { return m_deprecated; }
	


	/** additional set calls for new members **/
	void	setOptions(QString newValue)	{ m_options = newValue; }
	void	setForbidden(QString newValue)	{ m_forbidden = newValue; }
	void	setIgnore(QString newValue)	{ m_ignore = newValue; }
	void	setRestricted(QString newValue)	{ m_restricted = newValue; }
	void	setBroken(QString newValue)	{ m_broken = newValue; }
	void	setDeprecated(QString newValue)	{ m_deprecated = newValue; }
	
	

	void customEvent(QCustomEvent *e)
	{
		if (e->type()<1000)	/// not intended for this
			return;
		
		int signal = e->type() - 1000;

		switch(signal)
		{
			case 10: mutex[0].unlock(); emit T_findVulnaFinished(); break;
			case 0: mutex[1].unlock(); emit T_findOptionsFinished(); break;
			case 1: mutex[2].unlock(); emit T_findFilesFinished(); break;
			case 2: mutex[3].unlock(); emit T_findDescrFinished(); break;
			case 3: mutex[4].unlock(); emit T_exportPkgFinished(); break;
			case 4: mutex[5].unlock(); emit T_findFlagsFinished(); break;
		}


	}


signals:
	void	T_findVulnaFinished();
	void	T_findOptionsFinished();
	void	T_findFilesFinished();
	void	T_findDescrFinished();
	void	T_exportPkgFinished();
	void	T_findFlagsFinished();


};

/*class KMyProc : public KProcess
{
Q_OBJECT

	/** this is my own Proces class bacause all pre-made
	one lack one or the other feature. this is KProcess
	with the addition of canReadLine) functions (like in 
	QProcess). this is necessary because i dont want to 
	use signals and slots for output processing, but 
	instead use a while loop. the output is buffered.**
	
	private:
	
	QByteArray mStdoutBuffer;
	QByteArray mStderrBuffer;
	
//	QMutex	*m1, *m2;
	int count;
	public:
			
	KMyProc() : KProcess()
	{
		count = 0;
		mStdoutBuffer.clear();
		mStderrBuffer.clear();
//		m1 = new QMutex(this);
//		m2 = new QMutex(this);
		connect(this, SIGNAL(receivedStdout(KProcess*, char *, int)), this, SLOT(stdoutReceived(KProcess*, char *, int)));
		connect(this, SIGNAL(receivedStderr(KProcess*, char *, int)), this, SLOT(stderrReceived(KProcess*, char *, int)));
	}
			
	~KMyProc()
	{
		m1->unlock();
		m2->unlock();
//		delete m1;
//		delete m2;
	}
	
	bool 	canReadLineStdout() 		{ return (!mStdoutBuffer.isEmpty()); }
	bool 	canReadLineStderr() 		{ return (!mStderrBuffer.isEmpty()); }
	
	QString	readLineStdout()
	{
		/// reads first line from Buffer and then removes it so content "moves up"
		if (canReadLineStdout())// && ( m1->tryLock()))
		{

			QString Buffer = mStdoutBuffer.first();
			QStringList::Iterator it = mStdoutBuffer.begin();
			mStdoutBuffer.remove(*it);
//			m1->unlock();
			return (Buffer);
		} else 
			return ("");
	}
		
	QString	readLineStderr()
	{
		/// reads first line from Buffer and then removes it so content "moves up"
		if (canReadLineStderr())// && ( m2->tryLock()))
		{	
			QString Buffer = mStderrBuffer.first();
			QStringList::Iterator it = mStderrBuffer.begin();
			mStderrBuffer.remove(*it);
//			m2->unlock();
			return (Buffer);
		} else 
			return ("");
	}

	
	public slots:
			
	void	stdoutReceived(KProcess* proc, char *buffer, int buflen)
	{
//		m1->lock();	

		QStringList temp = QStringList::split('\n', QString::fromLatin1(buffer, buflen));
		for ( QStringList::Iterator it = temp.begin(); it != temp.end(); ++it ) 
        	{
			QString string = *it;
			mStdoutBuffer.append(string.remove(QChar(13)));
		}
//		m1->unlock();
	}
	void	stderrReceived(KProcess* proc, char *buffer, int buflen)		
	{

//		m2->lock();
		QStringList temp = QStringList::split('\n', QString::fromLatin1(buffer, buflen));
		for ( QStringList::Iterator it = temp.begin(); it != temp.end(); ++it ) 
        	{
			QString string = *it;
			mStderrBuffer.append(string.remove(QChar(13)));
		}	
//		m2->unlock();
	}

};
	**/
					
/*		
				
				
				
				
class QMyProc : public QProcess
{
Q_OBJECT
private:
	QMutex	m1;
public:

	QMyProc(QObject *parent, const char* name) : QProcess( parent, name) {}
	
	
	virtual bool start( QStringList *env=0 )
	{
//		qDebug("setsid: "+ QString::number(setsid()));
		
	
  		srand ( time(NULL) );
  
		return QProcess::start(env);
	}
		
	void stop();
};
*/
#endif
