/***************************************************************************
 *   Copyright (C) 2005, 2006 by Hannes Hauswedell                         *
 *   hannes.hauswedell@gmail.com                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "installwizard.h"
#include <kfiledialog.h>


/******************** install wizard***************************/
InstallWizard::InstallWizard(QWidget *parent, const char *name, int goal=1)
    :InstallWizardBase(parent, name)
{
	mParent = ((KPortsView*)parent);
	
	cancelButton()->setIconSet(mParent->IconLoader.loadIcon((QString)"button_cancel", KIcon::Small));
	backButton()->setIconSet(mParent->IconLoader.loadIcon((QString)"back", KIcon::Small));
	nextButton()->setIconSet(mParent->IconLoader.loadIcon((QString)"forward", KIcon::Small));
	finishButton()->setIconSet(mParent->IconLoader.loadIcon((QString)"button_ok", KIcon::Small));
	helpButton()->setIconSet(mParent->IconLoader.loadIcon((QString)"help", KIcon::Small));
	helpButton()->hide();
	bCommit->setIconSet(mParent->IconLoader.loadIcon((QString)"run", KIcon::Small));
	bKonsole->setIconSet(mParent->IconLoader.loadIcon((QString)"konsole", KIcon::Small));
//	bSavelog->setIconSet(mParent->IconLoader.loadIcon((QString)"filesave", KIcon::Small));

	bAdvancedOptions->hide();

	port = mParent->currentPort();


	
	binsource = 0; 
	aim 	= 0;
	recursive = 0;
	ignore 	= 0;
	show2arrow = false;

	mapper1 = 0;
	mapper2 = 0;
	logFile = 0;


	warningGiven = false;
	awaitingResults = false;
	
	backgroundColor = mParent->eSearch->paletteBackgroundColor();
	
	progress	= 0;
	forward		= true;
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(updateIcon() ));



	tOrigin2->setText(port->origin());
	tPixmap->setPixmap(*mParent->eIcon->pixmap());
	
	tOutput->setPaper(QBrush(QColor("black")));
	tOutput->setTextFormat(Qt::LogText);
	tOutput->setMaxLogLines(300); ///TODO: let it be set by user


	if (goal == 1)	//if this is not an uninstall
	{

		if (!port->avBinary())
		{
			rBinary->setChecked(false);
			rBinary->setEnabled(false);
			rBinary->setText(i18n("No binary Package available."));
			rSource->setChecked(true);
		}
		else
		{
			QString tmp;
			if ( !port->installed())
				tmp = i18n("Install of ");		
			else if ( port->installed()->version() < port->avBinary()->version() )
				tmp = i18n("Upgrade to ");
			else if ( port->installed()->version() > port->avBinary()->version() )
				tmp = i18n("Downgrade to ");
			else if ( port->installed()->version() == port->avBinary()->version() )
				tmp = i18n("Reinstall of ");
	
			tmp += port->avBinary()->version() + i18n(" via Package");

			if (QBsdPort::global->platform.pkgsystem != 0) ///when platform.pkgsystem is freebsd with portupgrade, mixed mode is recommended
				 tmp += i18n(" (recommended)");
	
			rBinary->setText(tmp);
			rBinary->setChecked(true);
		}
	
		if (!port->avSource())
		{
			rSource->setChecked(false);
			rSource->setEnabled(false);
			rSource->setText(i18n("No Package Source available."));
		}
		else
		{
			QString tmp;
			if ( !port->installed()	)
				tmp = i18n("Install of ");		
			else if ( port->installed()->version() < port->avSource()->version() )
				tmp = i18n("Upgrade to ");
			else if ( port->installed()->version() > port->avSource()->version() )
				tmp = i18n("Downgrade to ");
			else if ( port->installed()->version() == port->avSource()->version() )
				tmp = i18n("Reinstall of ");
	
			tmp += port->avSource()->version() + i18n(" via Source");
			
			rSource->setText(tmp);
		}
		
		if (!QBsdPort::global->platform.pkgsystem == 0)
		{
			rMixed->setChecked(false);
			rMixed->hide();
		} else if (port->avSource())
		{
			rMixed->setEnabled(true);
			rMixed->setChecked(true);

			QString tmp, ver;
			
			ver = port->avSource()->version();
			if (port->avBinary())
				if (port->avSource()->version() == port->avBinary()->version())
					ver = port->avBinary()->version();

			if ( !port->installed()	)
				tmp = i18n("Install of ");		
			else if ( port->installed()->version() < ver )
				tmp = i18n("Upgrade to ");
			else if ( port->installed()->version() > ver )
				tmp = i18n("Downgrade to ");
			else if ( port->installed()->version() == ver )
				tmp = i18n("Reinstall of ");
	
			tmp += ver + i18n(" in Mixed-Mode (recommended)");
			
			rMixed->setText(tmp);
		}


		if ( !rSource->isEnabled() && !rBinary->isEnabled() && !rMixed->isEnabled() )
			nextButton()->setDisabled(true);
	
		
/*		if (QBsdPort::global->platform.pkgsystem()==1)
		{
			cUseSrc->setEnabled(true);
			cUpgradeDeps->setEnabled(true);
			cUpgradeReqs->setEnabled(true);
			if (!source)
			{
				ePrefix->setEnabled(false);
				QToolTip::add( ePrefix, i18n( "Portupgrade does not support custom Prefix for binary installs. Disable Portupgrade to use this option or use source install." ) );
			} else
			{
				ePrefix->setEnabled(false);
				QToolTip::add( ePrefix, i18n( "Only change if you know what you are doing." ) );
			}
			
		}
		else
		{
			cUseSrc->setEnabled(false);
			cUpgradeDeps->setEnabled(false);
			cUpgradeReqs->setEnabled(false);
		}*/

	} else if (goal == 0)
	{

		aim = -1;

		groupMethod->hide();
		tMethodIcon->hide();

		//change the labels to refelct uninstall
		LCIntro->setText(i18n("<p align=center>You are about to remove this Port. You may edit Options on the following page. If you unsure, just leave them as they are.</p>"));

		setTitle(page(2), i18n("Step 3 - Uninstall"));

		setCaption(i18n("KPorts - Uninstall"));

		tPortName->setText("<b>" + port->installed()->name() + "</b>");
		tInfo->setText("<i>" + port->installed()->comment() + "</i>");
		tMethodIcon->setPixmap(mParent->IconLoader.loadIcon((QString)"trashcan_empty", KIcon::NoGroup));

		lOverview->removeColumn(2); /// hide "OPtions"
		lOverview->removeColumn(1); /// hide "nWarnings"
//		lResult->removeColumn(2);  /// hide "message"

//		tProgRemove->setText(i18n("Removal"));


		groupPrefix->hide();		
//		ePrefix->hide();
//		tPrefix->hide();

		cInstallReqs->hide();
		if (port->installed()->reqs() != "")
			cInstallReqs->setChecked(false);
		//is necessary for updateOverview

		cUpgradeDeps->setChecked(true);
		//need this for uninstalls 

		cUpgradeDeps->setText(i18n("remove"));
		cUpgradeReqs->setText(i18n("remove"));
		cIgnore->setText(i18n("force removal and risk a broken PkgDb"));
		QToolTip::add(cIgnore, i18n("It is strongly advised to remove all Dependencies aswell."));

		if (QBsdPort::global->platform.pkgsystem!=0)
			cUpgradeReqs->setEnabled(false);

		
			
		
	} else if (goal == 2)  /// upgrade everything
	{

			aim = -100;
			
			rSource->setChecked(true);
			rSource->setText(i18n("Upgrade via Sources"));
			rBinary->setText(i18n("Upgrade via Packages"));
			if (!QBsdPort::global->platform.pkgsystem == 0)
			{
				rMixed->setChecked(false);
				rMixed->hide();
			} else if (port->avSource())
			{
				rMixed->setEnabled(true);
				rMixed->setChecked(true);	
				rMixed->setText(i18n(" Upgrade in Mixed-Mode (recommended)"));
				
			}
				
			groupDeps->hide();
			groupReqs->hide();
				
			//TODO die erste seite noch gestalten + den rest dieses files anpassen
			
	}
			
	QListViewItem *tmpitem;
	if (port->installed())
		if ( port->installed()->deps()!="")
		{
/*			QStringList 	tmp;
			QString		tmp2;
	
			tmp = QStringList::split(' ', port->installed()->deps());
			
			for ( QStringList::Iterator it = tmpegin(); it != tmp.end(); it++ )
			{
				tmp2 = *it;
//				tmpport = QBsdPort::global->portWithName(tmp2, true, true);
//				mParent->pathToListViewItems(lRevDeps, tmpport->origin(), false, false);
				tmpitem = new QListViewItem(lDeps);
				tmpitem->setText(0,tmp2);
				tmpitem->setPixmap(0,mParent->IconLoader.loadIcon((QString)"package", KIcon::Small));
				
			}*/
		} else
		{
/*			tmpitem = new QListViewItem(lDeps);
			tmpitem->setPixmap(0,mParent->IconLoader.loadIcon((QString)"info", KIcon::Small));
			tmpitem->setText(0,i18n("No Dependencies"));
*/			groupDeps->setEnabled(false);
		}
	else
		groupDeps->hide();




	modeChanged(); //find out aim and set the vars

	connect(rSource, SIGNAL(stateChanged(int)), this, SLOT(saveASignal(int)));
	connect(rMixed, SIGNAL(stateChanged(int)), this, SLOT(saveASignal(int)));
	connect(rBinary, SIGNAL(stateChanged(int)), this, SLOT(saveASignal(int)));
	
	connect(cIgnore, SIGNAL(stateChanged(int)), this, SLOT(modeChanged()));
	connect(cUpgradeDeps, SIGNAL(stateChanged(int)), this, SLOT(modeChanged()));
	connect(cUpgradeReqs, SIGNAL(stateChanged(int)), this, SLOT(modeChanged()));


	connect(QBsdPort::global, SIGNAL(installStdOut(QString)), this, SLOT(stdOutReceived(QString)));
	connect(QBsdPort::global, SIGNAL(installErrOut(QString)), this, SLOT(stdErrReceived(QString)));
	connect(QBsdPort::global, SIGNAL(installCmd(QString)), this, SLOT(installCmdReceived(QString)));
	connect(QBsdPort::global, SIGNAL(installFinished()), this, SLOT(installProcFinished()));
		
	QTimer::singleShot(500, this, SLOT(resizeIcon()));
}

void InstallWizard::saveASignal(int newstate)
{
	/// when the current radiobutton in groupMethod changes two signals are emitted:
	/// stateChanged from both the old current and the new, we filter the old one out
	if (newstate==0)
		modeChanged();
	
}

void InstallWizard::resizeIcon()
{
	if (realBIN)
		tMethodIcon->setPixmap(mParent->IconLoader.loadIcon((QString)"binary", KIcon::NoGroup, groupMethod->height()));
	else
		tMethodIcon->setPixmap(mParent->IconLoader.loadIcon((QString)"source_cpp", KIcon::NoGroup, groupMethod->height()));
}


void InstallWizard::progression()
{
/*	if (forward)
	{
		progress+=3;
		if (progress>=75)
		{
			progress=75;
			forward=false;
		}
	} else
	{
		progress-=3;
		if (progress<=25)
		{
			progress=25;
			forward=true;
		}
	}*/
	
//	progressBar->setValue(progress+=5);
}

void InstallWizard::modeChanged()
{
	/** this is run every time major options change that effect the install process **/

	if (aim != -1 ) /// this is an install of any kind
	{
		QString version="";
		if (rSource->isChecked())
			binsource = 0;
		else if (rBinary->isChecked())
			binsource = 1;
		else if (rMixed->isChecked())
			binsource = 2;

		if (binsource==0) /// source only
			realBIN = false;
		else if (binsource==1) /// binary only
			realBIN = true;
		else if (binsource==2)
		{
			realBIN =false;
			if (port->avBinary())
				if (port->avSource()->version() == port->avBinary()->version()) /// portupgrade only resorts to binary if it equals source version
					realBIN = true;
		}

		if (realBIN)
		{
			tPortName->setText("<b>" + port->avBinary()->name() + "</b>");
			tInfo->setText("<i>" + port->avBinary()->comment() + "</i>");
			version = port->avBinary()->version();
			tMethodIcon->setPixmap(mParent->IconLoader.loadIcon((QString)"binary", KIcon::NoGroup, groupMethod->height()));
		} else
		{
			tPortName->setText("<b>" + port->avSource()->name() + "</b>");
			tInfo->setText("<i>" + port->avSource()->comment() + "</i>");
			version = port->avSource()->version();
			tMethodIcon->setPixmap(mParent->IconLoader.loadIcon((QString)"source_cpp", KIcon::NoGroup, groupMethod->height()));
		}
		
		groupDeps->show(); //make sure they are there
				
		QToolTip::remove(cUpgradeReqs);
	
		if ( !port->installed())
		{
			aim = 0;  //fresh install
			groupDeps->hide(); //new ports have no dependencies
			setTitle(page(2), i18n("Step 3 - Install     Overview"));
			
		} else if ( port->installed()->version() < version )
		{
			aim = 1;  //upgrade
			setTitle(page(2), i18n("Step 3 - Upgrade     Overview"));
			
		} else if ( port->installed()->version() > version )
		{
			aim = 2;  //downgrade
			setTitle(page(2), i18n("Step 3 - Downgrade     Overview"));
			QToolTip::add(cUpgradeReqs, i18n("On Downgrades/Reinstalls this option reinstalls all Rqeuirements."));
		} else if ( port->installed()->version() == version )
		{
			aim = 3;  //reinstall
			setTitle(page(2), i18n("Step 3 - Reinstall     Overview"));
			QToolTip::add(cUpgradeReqs, i18n("On Downgrades/Reinstalls this option reinstalls all Rqeuirements."));
		}
	
		
		if (QBsdPort::global->platform.pkgsystem==0)
		{
			cUpgradeDeps->setEnabled(true);
			cUpgradeReqs->setEnabled(true);
			if (binsource!=0)
			{
				ePrefix->setEnabled(false);
				QToolTip::add( ePrefix, i18n( "Portupgrade does not support custom Prefix for binary installs. Disable Portupgrade to use this option or use source install." ) );
			} else
			{
				ePrefix->setEnabled(true);
				QToolTip::add( ePrefix, i18n( "Only change if you know what you are doing." ) );
			}
		} else
		{
			cUpgradeDeps->setEnabled(false);
			cUpgradeReqs->setEnabled(false);
		}
	
	
	
		
	}


	if (cIgnore->isChecked())
		ignore=1;
	else
		ignore=0;

	if ( (cUpgradeReqs->isChecked()) && (cUpgradeDeps->isChecked()) ) 
		recursive = 3;
	else if (cUpgradeReqs->isChecked())
		recursive = 2;
	else if (cUpgradeDeps->isChecked())
		recursive = 1;
	else
		recursive = 0;


	QBsdPort	*tmpport;

	QString reqq(""), pref("");

	if (aim!=-1)
	{
		if (realBIN)
		{
			reqq = port->avBinary()->reqs();
			pref = port->avBinary()->prefix();
		} else
		{
			reqq = port->avSource()->reqs();
			pref = port->avSource()->prefix();
		}
	
	
		if (KPortsView::g_sPrefix=="")
			ePrefix->setURL(pref);
		else
			ePrefix->setURL(KPortsView::g_sPrefix);
	} else
		reqq = port->installed()->reqs();

	QListViewItem *tmpitem;
	if  (reqq=="") 
	{
/*		tmpitem = new QListViewItem(lReqs);
		tmpitem->setPixmap(0,mParent->IconLoader.loadIcon((QString)"info", KIcon::Small));
		tmpitem->setText(0,i18n("No Requirements"));
*/
		groupReqs->setEnabled(false);

	}
	else
	{
/*		QStringList 	tmp;
		QString		tmp2;

		tmp = QStringList::split(' ', reqq);
		
		lReqs->clear();
		
		for ( QStringList::Iterator it = tmp.begin(); it != tmp.end(); it++ )
		{
			tmp2 = *it;
//			tmp2 = tmp2.section('-',0,-2);
			tmpitem = new QListViewItem(lReqs);
			tmpitem->setText(0, tmp2);
			tmpitem->setPixmap(0, mParent->IconLoader.loadIcon((QString)"package", KIcon::Small));
		}
*/
		

	}

	if  ( (aim != -1 ) && (reqq=="") && (!groupDeps->isVisible()) )
		cIgnore->setEnabled(false);



//	if (!depmasterthread->isActive())
		updateOverview();
}

/// in uninstall either RemoveDeps or keep going have to be set
void InstallWizard::cUpgradeDepsToggled()
{
	if ( (!cUpgradeDeps->isChecked()) && (aim==-1) )
		cIgnore->setChecked(true);
}

void InstallWizard::cIgnoreToggled()
{
	if ( (!cIgnore->isChecked())  && (aim==-1) )
		cUpgradeDeps->setChecked(true);
}

void InstallWizard::CommitPressed()
{
	if (!backButton()->isEnabled())
	{
		if (geteuid()!=0)
		{
			KMessageBox::error(this, i18n("The Process cannot be aboorted, because you do not have sufficient "
							"previliges. You should start KPorts as root to avoid this."));
			return;
		}
		
		if (KMessageBox::questionYesNo(this, i18n("Are you sure that you wish to abort the process?"), QString(),
			KStdGuiItem::yes(), KStdGuiItem::no(),  "dontAskAbort") == KMessageBox::No)
			return;
		
		bCommit->setEnabled(false);
		if (aim>=0)
			QBsdPort::global->stopProc("installProc");
		else
			QBsdPort::global->stopProc("uninstallProc");
			
		return;
	}

	if (cLog->isChecked())
	{
		logFile = new QFile(eLogfile->url());
		
		if (!logFile->open(IO_WriteOnly))
		{
			if (KMessageBox::warningContinueCancel(this, i18n("Could not open Logfile for writing. check your permissions "
									" or choose another file. If you choose to continue, no Log will be created."))
				!= KMessageBox::Continue)
			{
				cLog->setChecked(false);
				return;
			}
		}
	
	
	}


	backButton()->setEnabled(false);
	bCommit->setText(i18n("abort"));
	
	bCommit->setIconSet(mParent->IconLoader.loadIcon((QString)"cancel", KIcon::Small));



	tOutput->clear();
	wasInterupted = 0;


	if (aim>=0) /// install
	{		
		QString Prefix="", comPrefix="";
		
		if (realBIN)
			comPrefix = port->avBinary()->prefix();
		else
			comPrefix = port->avSource()->prefix();
	
	
		QString Env(""); /// environment vars for the install
		
		/// check if different prefix was specified
		if (ePrefix->isEnabled())
			if (ePrefix->url() != comPrefix)
				Env = ePrefix->url();
				
		/// get the non-default options from the individual ports
		for ( int i = 0; i < portsInView.count(); ++i)
		{
			if (portsInView.at(i)->env != "")
				Env += portsInView.at(i)->env;
		}
			
		//QMessageBox::information(this, "Install Command", port->install_cmd(aim, recursive, binsource, ignore, Env), QMessageBox::Ok);
				
		QBsdPort::global->T_install_port(port, aim, recursive, binsource, ignore, Env);
		setTitle(page(2), i18n("Step 3 - Install     Progress"));
	}else
	{
		setTitle(page(2), i18n("Step 3 - Uninstall     Progress"));
		QBsdPort::global->T_uninstall_port(port, recursive, ignore);
	}


	lOverview->removeColumn(2);
	lOverview->removeColumn(1);
	lOverview->insertColumns(lOverview->numCols());
	lOverview->horizontalHeader()->setLabel(lOverview->numCols()-1, i18n("Status"));
	
	tExtra->setText(i18n("<p align=\"center\">Preparing...</p>"));
	
	timer->start(500);
	

	
}


void InstallWizard::addPortToOverview(QBsdPort *Port, int type)
{
	if (!Port->avSource() && !Port->avBinary())
		return;
	
	QString bigversion, term;
	QPixmap icon;
	bool	bin=true;

	if (aim>=0) /// this is an install
	{
	
		if (binsource==0)
			bin = false;
		else if (binsource==1)
		{
			bin = true;	
			if (!Port->avBinary()) /// is there a binary package for this port
			{
				if ((Port->installed() && (type == 1) && (cUpgradeReqs->isChecked()) )
					|| (Port->installed() && (type == 2) && (cUpgradeDeps->isChecked()))) /// if it would be an upgrade
				{
					if (Port->avSource())
						if (Port->avSource()->version() > Port->installed()->version())
							KMessageBox::information(this, i18n("A binary Package for ") + Port->origin() 
							+ i18n(" is not available.\nThe dependency/requirement will not be upgraded "
							"(although a newer version is available as source)."), i18n("Kports - Warning")); /// just warn
				} else if ((!Port->installed() && (type == 1))) /// if these are needed reqs show critical
				{	
					if (QBsdPort::global->platform.pkgsystem!=0)
						KMessageBox::error(this, i18n("A binary Package for ") + Port->origin() 
							+ i18n(" is not available,\ntherefor the requirement cannot be met.\nYou should use Source Install instead. ") );
					else
						KMessageBox::error(this,i18n("A binary Package for ") + Port->origin() 
							+ i18n(" is not available,\ntherefor the requirement cannot be met.\nYou should use Mixed Install instead. "));
				}
				return;
			}
		} else if (binsource==2)
		{
			bin = false;
			if (Port->avBinary())
				if (Port->avSource()->version() == Port->avBinary()->version()) /// portupgrade only resorts to binary in -P mode if it equals source version
					bin = true;
		}
	
		if (bin)
			bigversion = Port->avBinary()->version();
		else
			bigversion = Port->avSource()->version();
	
	//	std::cout << Port->name()<< endl;
		
		if (!Port->installed()) /// fresh install
		{
			icon = mParent->IconLoader.loadIcon((QString)"forward", KIcon::Small);  
			term = i18n("install");
			if (bin)
				term += "(bin) of " + Port->avBinary()->version();
			else
				term += "(src) of " + Port->avSource()->version();
			/// needs no return condition because 1. main port and reqs will always be installed
			/// if not already there and deps are always already there 
		}
		else if ( Port->installed()->version() < bigversion ) /// upgrade
		{
			icon = mParent->IconLoader.loadIcon((QString)"up", KIcon::Small); 
			term = i18n("upgrade");
			if (bin)
				term += "(bin) to " + Port->avBinary()->version();
			else
				term += "(src) to " + Port->avSource()->version();
			if ( (type == 1) && (!cUpgradeReqs->isChecked()) )
				return;
			if ( (type == 2) && (!cUpgradeDeps->isChecked()) )
				return;
		}
		else if ( Port->installed()->version() > bigversion ) /// downgrade
		{
			icon = mParent->IconLoader.loadIcon((QString)"down", KIcon::Small);  
			term = i18n("downgrade");
			if (bin)
				term += "(bin) to " + Port->avBinary()->version();
			else
				term += "(src) to " + Port->avSource()->version();
			if ( (type == 1) && (!cUpgradeReqs->isChecked()) )
				return;
			if ( (type == 2) && (!cUpgradeDeps->isChecked()) )
				return;
			if ( (aim<=1) )	//force option not set
				return;
		}
		else if ( Port->installed()->version() == bigversion ) /// reinstall
		{
			icon = mParent->IconLoader.loadIcon((QString)"reload", KIcon::Small); 
			term = i18n("reinstall");
			if (bin)
				term += "(bin) of " + Port->avBinary()->version();
			else
				term += "(src) of " + Port->avSource()->version();
			if ( (type == 1) && (!cUpgradeReqs->isChecked()) )
				return;
			if ( (type == 2) && (!cUpgradeDeps->isChecked()) )
				return; 
			if ( (aim<=1)  )	//force option not set
				return;
		}
	
		
	} else /// uninstall
	{

		if ( (type == 1) && (!cUpgradeReqs->isChecked()) )
			return;
		if ( (type == 2) && (!cUpgradeDeps->isChecked()) )
			return; 

		icon = mParent->IconLoader.loadIcon((QString)"trashcan_empty", KIcon::Small);  //downgrade
		term = i18n("uninstall of ") + Port->installed()->version();
	}

		
	lOverview->insertRows(lOverview->numRows());/// insert one row at the end
	lOverview->setRowStretchable(lOverview->numRows()-1, false);
	lOverview->verticalHeader()->setLabel(lOverview->numRows()-1, Port->avSource()->name());
	lOverview->setText(lOverview->numRows()-1,0 , term);
	lOverview->setPixmap(lOverview->numRows()-1,0 , icon);
	lOverview->setRowReadOnly(lOverview->numRows()-1, true);
			
	_portInView *entry = new _portInView;
	entry->p = Port;
	entry->bin = bin;
	entry->line = lOverview->numRows()-1;
	entry->finished = false;
	entry->errors = "";
	entry->env = "";
	/// save some info in this stringlist for later reference
	portsInView.append(entry);
			
	lOverview->adjustColumn(0);
	


	if ( (aim!=-1)  )
	{
		/// uninstall doesnt have these columns
		lOverview->adjustColumn(1);
		lOverview->adjustColumn(2);

		if (bin)
		{
			if ( (Port->avBinary()->vulna()=="no_init") && (QBsdPort::global->portaudit) )
			{
				connect(Port->avBinary(), SIGNAL(T_findVulnaFinished()), mapper1, SLOT(map()));
				mapper1->setMapping(Port->avBinary(), lOverview->numRows()-1);
				Port->avBinary()->T_findVulna();
			}
			
			lOverview->setText(lOverview->numRows()-1,2 , i18n("-")); /// binary pkgs have no options
		}
			
			
		if (!bin)
		{
			if ( (Port->avSource()->vulna()=="no_init") && (QBsdPort::global->portaudit) )
			{
				connect(Port->avSource(), SIGNAL(T_findVulnaFinished()), mapper1, SLOT(map()));
				Port->avSource()->T_findVulna();
			}	
			
			if (Port->avSource()->ignore()=="no_init")
			{
				connect(Port->avSource(), SIGNAL(T_findFlagsFinished()), mapper1, SLOT(map()));
				Port->avSource()->T_findFlags();
			}
			mapper1->setMapping(Port->avSource(), lOverview->numRows()-1); /// only one mapping
			
			if (Port->avSource()->options()=="no_init")
			{
				connect(Port->avSource(), SIGNAL(T_findOptionsFinished()), mapper2, SLOT(map()));
				Port->avSource()->T_findOptions();
				mapper2->setMapping(Port->avSource(), lOverview->numRows()-1);
			} else
				addOptions(lOverview->numRows()-1);
		}
		
		addWarning(lOverview->numRows()-1);
			

	}

}

				
void InstallWizard::addWarning(int row)
{

//	lState->setText(portsInView[row]);
	QBsdPort *Port = portsInView.at(row)->p;
	int bin = portsInView.at(row)->bin;
			
	if (!Port)
	{
		std::cout << "wwaahhhhhh\n";
		return;
	}
	
	if (bin)
	{

		/// is all info fetched?
		if ( (Port->avBinary()->vulna()=="no_init") && (QBsdPort::global->portaudit) )
			return; 
		/// are there any warnings?
		if ( 	(Port->avBinary()->vulna()=="") || (!QBsdPort::global->portaudit) )
		{
			if (!Port->installed())
			{
				lOverview->setText(row, 1 , i18n("none"));
				return;
			}
			else if (Port->installed()->updWarning()=="")
			{
				lOverview->setText(row, 1 , i18n("none"));
				return;
			}
		}
	}
	

	if (!bin)
	{
//		lState->setText(portsInView[row]+" vulna:"+Port->avSource()->vulna());
		/// is all info fetched?
		if ( (Port->avSource()->vulna()=="no_init") && (QBsdPort::global->portaudit) )
			return;	
		
		if (Port->avSource()->deprecated()=="no_init")
			return;
		/// are there any warnings?
		if ( (	(Port->avSource()->vulna()=="") 	||
			(!QBsdPort::global->portaudit)	)	&&
			(Port->avSource()->ignore()=="") 	&&
			(Port->avSource()->deprecated()=="") 	&&
			(Port->avSource()->restricted()=="") 	&&
			(Port->avSource()->broken()=="") 	&&
			(Port->avSource()->forbidden()=="") 	) 
		{
			if (!Port->installed())
			{
				lOverview->setText(row, 1 , i18n("none"));
				return;
			} else if (Port->installed())
			{
				if (Port->installed()->updWarning()=="")
				{
					lOverview->setText(row, 1 , i18n("none"));
					return;
				}
			}
		} 
		
	}
	
//	lOverview->setText(row, 1 , i18n("YES"));
	
	
	QPortLink *item = new QPortLink(this,((QString)"pWarning"+Port->name()), Port);
	item->setText(i18n("view"));
	item->setPaletteBackgroundColor(backgroundColor);

	lOverview->setCellWidget(row, 1, item);
	connect(item, SIGNAL(clickedPort(QBsdPort*)), this, SLOT(warningPushed(QBsdPort*)));	
	
}
void InstallWizard::addOptions(int row)
{
	
	QBsdPort *Port = portsInView.at(row)->p;
			
	if (!Port)
	{
		std::cout << "wwaahhhhhh\n";
		return;
	}
	
	if (!Port->avSource())
	{
		lOverview->setText(row, 2 , i18n("-"));
		return;
	}
	
//	QMessageBox::information(this, Port->avSource()->name(), Port->avSource()->options(), QMessageBox::Ok);
	
	if (Port->avSource()->options()=="")
	{
		lOverview->setText(row, 2 , i18n("none"));
		return;
	} else
	{
		QPortLink *item = new QPortLink(this,((QString)"pOptions"+Port->name()), Port);
		item->setText(i18n("edit"));
		item->setPaletteBackgroundColor(backgroundColor);
	
		lOverview->setCellWidget(row, 2, item);
		connect(item, SIGNAL(clickedPort(QBsdPort*)), this, SLOT(optionsPushed(QBsdPort*)));	
	}
	

}

void InstallWizard::warningPushed(QBsdPort *Port)
{
	bool bin = false;
	/// find out if binary orsource is used
	if (binsource==0)
		bin = false;
	else if (binsource==1)
		bin = true;	
	else if (binsource==2)
	{
		bin = false;
		if (Port->avBinary())
			if (Port->avSource()->version() == Port->avBinary()->version()) /// portupgrade only resorts to binary in -P mode if it equals source version
				bin = true;
	}
	
	
	PortWarningDlg *dialog = new PortWarningDlg(this);

	/// be default stuff is hidden
	dialog->updGroup->hide();
	dialog->flagGroup->hide();
	dialog->secGroup->hide();
	dialog->cBroken->hide();
	dialog->cIgnore->hide();
	dialog->cDeprecated->hide();
	dialog->cRestricted->hide();
	dialog->cForbidden->hide();

	/// check for updating warning
	
	if (Port->installed())
		if (Port->installed()->updWarning()!="")
		{
			dialog->updGroup->show();
			dialog->eUpdating->setText(Port->installed()->updWarning());
		}

	/// check fo vulnas
	QString vulna("");
	if (!bin)
		vulna = Port->avSource()->vulna();
	else
		vulna = Port->avBinary()->vulna();
		
	if ( (vulna!="") && (vulna!="no_init") )
	{
		dialog->secGroup->show();
		
		QStringList vulnalist = QStringList::split("\n", vulna);
		QString genervt("");
		for ( QStringList::Iterator it = vulnalist.begin(); it != vulnalist.end(); ++it)
		{ 
			genervt = *it;
			KURLLabel *label = new KURLLabel(dialog->secGroup);
			label->setText(genervt.section('|',0,0).section(" -- ", 1, 1));
			label->setURL(genervt.section('|',1,1));
			label->show();
			label->setHighlightedColor(QString("red"));
			label->setSelectedColor(QString("red"));
			label->setUnderline(false);
			label->setGlow(false);
			label->setFloat(true);
			dialog->secGroup->layout()->activate();
			connect(label, SIGNAL(leftClickedURL(const QString&)), mParent, SLOT(URLClicked(const QString& )));
			
			QToolTip::add(label, i18n("Click for more Information"));
		}
	}
	
	/// check for flags
	if (!bin)
	{
		if (Port->avSource()->ignore()!="")
		{
			dialog->flagGroup->show();
			dialog->cIgnore->show();
			dialog->cIgnore->setText(i18n("IGNORE: " + Port->avSource()->ignore()));
		}
	
		if (Port->avSource()->deprecated()!="")
		{
			dialog->flagGroup->show();
			dialog->cDeprecated->show();
			dialog->cDeprecated->setText(i18n("DEPRECATED: " + Port->avSource()->deprecated()));
		}
				
		if (Port->avSource()->broken()!="")
		{
			dialog->flagGroup->show();
			dialog->cBroken->show();
			dialog->cBroken->setText(i18n("BROKEN: " + Port->avSource()->broken()));
		}
				
		if (Port->avSource()->restricted()!="")
		{
			dialog->flagGroup->show();
			dialog->cRestricted->show();
			dialog->cRestricted->setText(i18n("RESTRICTED: " + Port->avSource()->restricted()));
		}
				
		if (Port->avSource()->forbidden()!="")
		{
			dialog->flagGroup->show();
			dialog->cForbidden->show();
			dialog->cForbidden->setText(i18n("FORBIDDEN: " + Port->avSource()->forbidden()));
		}
	
	
	}

	dialog->resize( dialog->minimumSize() );
				
	/** and show it **/
	dialog->exec();
//	QMessageBox::information(this, "test","has warnings", QMessageBox::Ok);

}

void InstallWizard::optionsPushed(QBsdPort *Port)
{
	if (!Port)
		return;
	if (!Port->avSource())
		return;
	if (Port->avSource()->options()=="")
		return;
	
	/// find the row this port is listed in
	QBsdPort *tmpport, tmpstring;
	int row = 0;
	for ( int i = 0; i < portsInView.count(); ++i)
	{
		tmpport = portsInView.at(i)->p;
		if (tmpport == Port)
		{
			row = i;
			i = portsInView.count(); /// we can quit the loop
		}
	}
	
	QString tmp(""), result("");
	PortOptionsDlg *dialog = new PortOptionsDlg(this, "PortOptionsDlg");
	
//	qDebug("OPTIONS: " + Port->avSource()->options());
//	qDebug("PORTSINVIE: " + portsInView[row]);
			
	/// iterate through the options and add checkboxes
	QStringList list = QStringList::split("\n", Port->avSource()->options());
	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it)
	{ 
		tmp = *it;
		QCheckBox *checker = new QCheckBox(dialog->optionsGroup);
		checker->setText(tmp.section('|', 1,1));
		/// set states for checkboxes
		if ( portsInView.at(row)->env.contains(" WITH_"+tmp.section('|',0,0).remove(' ')+"=1 ") 
			|| portsInView.at(row)->env.contains(" WITHOUT_"+tmp.section('|',0,0).remove(' ')+"=1 ") )
		{
			/// state was modified -> is opposite of default
			if (tmp.section('|', 2,2) == "on")
				checker->setChecked(false);
			else
				checker->setChecked(true);
		} else
		{
			/// set state to default
			if (tmp.section('|', 2,2) == "on")
				checker->setChecked(true);
			else
				checker->setChecked(false);
		}
		checker->show();
		dialog->optionsGroup->layout()->activate();
	}
	
	dialog->resize( dialog->minimumSize() );
	
	/// run the dialog
	dialog->exec();
	
	/// after dialog is closed, check which options differ from default
	QObjectList *l = dialog->optionsGroup->queryList( "QCheckBox" );
	QObjectListIt it( *l ); // iterate over the buttons
	QObject *obj;
	QCheckBox *cobj;
	int i = 0;
	while ( (obj = it.current()) != 0 )  /// we parallely iterate through options-string and objectlist
	{
		++it;
		cobj = (QCheckBox*)obj;
		tmp = list[i];
		if ( (cobj->isChecked()) && (tmp.section('|', 2,2) == "off") ) 
			result += " WITH_" + tmp.section('|',0,0).remove(' ') + "=1" ;
		else if ( (!cobj->isChecked()) && (tmp.section('|', 2,2) == "on") )
			result += " WITHOUT_" + tmp.section('|',0,0).remove(' ') + "=1" ;
		++i;
	}
	
	/// add the options that differ from default the item in the list
	portsInView.at(row)->env = result + " ";
	
}
				
void InstallWizard::updateOverview()
{
	QStringList tmp;
	QString tmp2, dep, all="";
	QBsdPort *tmpport;
	
	currentRow = -1;
	
	wasInterupted = 0;
	tExtra->setText("");
	awaitingResults=false;

	/// warning mapper
	if (mapper1)
		mapper1->disconnect(this);
	delete mapper1;
	mapper1 = new QSignalMapper(this);
	connect( mapper1, SIGNAL( mapped( int ) ), this, SLOT( addWarning(int) ) );
	
	/// options mapper
	if (mapper2)
		mapper2->disconnect(this);
	delete mapper2;
	mapper2 = new QSignalMapper(this);
	connect( mapper2, SIGNAL( mapped( int ) ), this, SLOT( addOptions(int) ) );

	while (lOverview->numRows()>0)
		lOverview->removeRow(0);

	while (lOverview->numCols()>0)
		lOverview->removeColumn(0);
	
	lOverview->insertColumns(lOverview->numCols());
	lOverview->horizontalHeader()->setLabel(lOverview->numCols()-1, i18n("Action"));
	if (aim != -1)
	{
		lOverview->insertColumns(lOverview->numCols());
		lOverview->horizontalHeader()->setLabel(lOverview->numCols()-1, i18n("Warnings"));
		lOverview->insertColumns(lOverview->numCols());
		lOverview->horizontalHeader()->setLabel(lOverview->numCols()-1, i18n("Options"));
	}
	
	portsInView.clear();


	for (int i=0;i<=2;i++)
	{
		dep = "";
		switch(i)
		{
			/// the conditions at case 1 and 2 are not really necessary since
			/// they are checkeed in addPortToOverview again
			case 0:
				dep = port->origin();
				break;
			case 1: if (cUpgradeReqs->isChecked() || cInstallReqs->isChecked())
				{
					if (realBIN)
						dep = port->avBinary()->reqs();
					else
						dep = port->avSource()->reqs();
				}
				break;
			case 2: if (cUpgradeDeps->isChecked())
					dep = port->installed()->deps(); 
				break;
		}
//		lState->setText("processing dep:"+dep);
//		std::cout << "processing dep:"<< dep.latin1()<<endl;
		/// we will now parse the list of reqs or deps (or simplay add main port in case of i=0)
		if (dep!="")
		{
	
			tmp = QStringList::split(' ', dep);
	
			for ( QStringList::Iterator it = tmp.begin(); it != tmp.end(); it++ )
			{
				tmp2 = *it;
				if (i == 1) /// requirements are in name-version form
				{
					if ((tmpport = QBsdPort::global->portWithName(tmp2.section('-',0,-2), 3, tmp2.section('-',-1,-1))) == NULL) /// check installed first
						tmpport = QBsdPort::global->portWithName(tmp2.section('-',0,-2), 1, tmp2.section('-',-1,-1)); 
				} else if (i == 2) /// dependencies are in name-version form, and we know they are installed
					tmpport = QBsdPort::global->portWithName(tmp2.section('-',0,-2), 3, tmp2.section('-',-1,-1)); 
				else /// add main port
					tmpport = QBsdPort::global->portWithOrigin(tmp2);
				
				
				if (tmpport != 0)
//				{
//					lState->setText("port hat origin:"+tmp2+ "+ JA");
					addPortToOverview(tmpport, i);
//				} else
//					lState->setText("port hattte origin:"+tmp2+ "+ NEIN");
			}
		}
	}
	
	

}
void InstallWizard::bViewDepsClicked()
{
	if (!port->installed())
		return;
	
	QDialog *dlg = new QDialog(this);
					

	QGridLayout *Form1Layout = new QGridLayout( dlg, 1, 1, 11, 6, "Form1Layout"); 
	
	QLabel *label = new QLabel( dlg, "label" );
	
	Form1Layout->addWidget( label, 0, 0 );
	
	QListView *list = new QListView( dlg, "list" );
	list->addColumn( i18n( "Name" ) );
	list->addColumn( i18n( "Installed" ) );
	list->addColumn( i18n( "av Binary" ) );
	list->addColumn( i18n( "av Source" ) );
	list->setResizePolicy( QListView::AutoOneFit );
	list->setSelectionMode( QListView::NoSelection );
	list->setAllColumnsShowFocus( TRUE );
	list->setResizeMode( QListView::AllColumns );
	
	Form1Layout->addWidget( list, 1, 0 );

	dlg->setCaption( i18n( "KPorts - Dependencies for " ) + port->origin());
		
	label->setText( i18n( "This Port is required by:" ) );
		
	QString Deps("");
		
	Deps = port->installed()->deps();

		
	if (Deps!="")
	{
		QStringList 	tmp = QStringList::split(' ', Deps);
		QString 	tmp2;
		QListViewItem	*tmpitem;
		QBsdPort	*tmpport;
		for ( QStringList::Iterator it = tmp.begin(); it != tmp.end(); it++ )
		{
			tmp2 = *it;
			tmpport = QBsdPort::global->portWithName(tmp2.section('-',0,-2), 3, tmp2.section('-',-1,-1));
			if (!tmpport)
			{
				KMessageBox::information(this, tmp2 + i18n(" cannot ne found."));
			} else
			{
			
				tmpitem = new QListViewItem(list);
				tmpitem->setText(0, tmp2.section('-',0,-2));
				tmpitem->setPixmap(0,mParent->IconLoader.loadIcon((QString)"package", KIcon::Small));	
				if (tmpport->installed())
					tmpitem->setText(1, tmpport->installed()->version());
				if (tmpport->avBinary())
					tmpitem->setText(2, tmpport->avBinary()->version());
				if (tmpport->avSource())
					tmpitem->setText(3, tmpport->avSource()->version());
			}
					
		}
	}

	dlg->resize( QSize(333, 301).expandedTo(minimumSizeHint()) );
//	dlg->clearWState( WState_Polished );
	dlg->show();
	
}

					
					
void InstallWizard::bViewReqsClicked()
{
	QDialog *dlg = new QDialog(this);
					

	QGridLayout *Form1Layout = new QGridLayout( dlg, 1, 1, 11, 6, "Form1Layout"); 
	
	QLabel *label = new QLabel( dlg, "label" );
	
	Form1Layout->addWidget( label, 0, 0 );
	
	QListView *list = new QListView( dlg, "list" );
	list->addColumn( i18n( "Name" ) );
	list->addColumn( i18n( "Installed" ) );
	list->addColumn( i18n( "av Binary" ) );
	list->addColumn( i18n( "av Source" ) );
	list->setResizePolicy( QListView::AutoOneFit );
	list->setSelectionMode( QListView::NoSelection );
	list->setAllColumnsShowFocus( TRUE );
	list->setResizeMode( QListView::AllColumns );
	
	Form1Layout->addWidget( list, 1, 0 );

	dlg->setCaption( i18n( "KPorts - Requirements for " ) + port->origin());
		
	label->setText( i18n( "This Port requires:" ) );
		
	QString Reqs("");
	if (realBIN)
		Reqs = port->avBinary()->reqs();
	else
		Reqs = port->avSource()->reqs();
		
	if (Reqs!="")
	{
		QStringList 	tmp = QStringList::split(' ', Reqs);
		QString 	tmp2;
		QListViewItem	*tmpitem;
		QBsdPort	*tmpport;
		for ( QStringList::Iterator it = tmp.begin(); it != tmp.end(); it++ )
		{
			tmp2 = *it;
			tmpport = QBsdPort::global->portWithName(tmp2.section('-',0,-2), 0, tmp2.section('-',-1,-1));
			if (!tmpport)
			{
				KMessageBox::information(this, tmp2 + i18n(" cannot be found."));

			} else
			{
				tmpitem = new QListViewItem(list);
				tmpitem->setText(0, tmp2.section('-',0,-2));
				tmpitem->setPixmap(0,mParent->IconLoader.loadIcon((QString)"package", KIcon::Small));	
				if (tmpport->installed())
					tmpitem->setText(1, tmpport->installed()->version());
				if (tmpport->avBinary())
					tmpitem->setText(2, tmpport->avBinary()->version());
				if (tmpport->avSource())
					tmpitem->setText(3, tmpport->avSource()->version());
			}
					
		}
	}

	dlg->resize( QSize(333, 301).expandedTo(minimumSizeHint()) );
//	dlg->clearWState( WState_Polished );
	dlg->show();
}
					
void InstallWizard::showKonsole(bool yes)
{
	if (yes)
		wsDetails->raiseWidget(1);
	else
		wsDetails->raiseWidget(0);
}
					
					

void InstallWizard::stdOutReceived(QString Buffer)
{

	/// appen to textedit
	tOutput->append(Buffer);
	///and to file
	if (logFile)
		if (logFile->isOpen())
			logFile->writeBlock(Buffer + '\n', Buffer.length()+1);


	/** interpret output **/
	if (aim>=0)
	{
		switch (QBsdPort::global->platform.pkgsystem)
		{
			case -1: KMessageBox::information(this, i18n("No simplified Progress information is calculated when Portupgrade is not used."
									"Use Portupgrade or klick on \"Konsole\" to view unfiltered output."));
				break;
			case 0:{ /// freebsd ports with porupgrade
	
				if (Buffer.startsWith("--->  Session started")) /// starting 
				{
					currentRow = 0;
//					mParent->app->changeStatusbar("session started");
				} else if (Buffer.contains("Build of")) /// a  port is build is started or finished
				{
					/// do nothing IMPORTANT DONT CHANGE THIS	
				} else if (Buffer.contains("Cleaning for") ) /// a  port is building
				{
					lOverview->setText(currentRow,1, i18n("checking Reqs"));
				} else if (Buffer.contains("Building for") || Buffer.contains("Configuring for")) /// a  port is building
				{
									/// make sure this is actually displayed as source install/upgrade
					if (!lOverview->text(currentRow, 0).contains("(src)"))
					{
						portsInView.at(currentRow)->bin = 0;
						lOverview->setText(currentRow, 0, lOverview->text(currentRow, 0).replace("(bin)", "(src)"));
					}
					
					lOverview->setText(currentRow,1, i18n("building"));
//					mParent->app->changeStatusbar("building");
				} else if (Buffer.contains("started at:") || /// the next port is processed (src)
						(Buffer.contains("Verifying install for")) || /// new dependency on src install
						(Buffer.contains("Returning to build of")) )
				{
					//int newRow = -1;
					QBsdPort *tmpport = 0;
					QStringList list;
					if (!Buffer.contains("Verifying"))
					{
						Buffer.remove("--->"); /// so grep for '-' wil work 
						list = QStringList::split(' ', Buffer);
						if (Buffer.contains("/")) /// general or source notice
							tmpport = QBsdPort::global->portWithOrigin(list.grep("/").first());
						else /// name-version form given
							tmpport = QBsdPort::global->portWithName(list.grep("-").first().section('-',0,-2), 0, list.grep("-").first().section('-',-1,-1));
					} else
					{
						list = QStringList::split(' ', Buffer);
						tmpport = QBsdPort::global->portWithOrigin(list.grep("/").first().section('/',3,4).remove(' '));
	
					}
				
					
					
					
					if (tmpport)
					{
//						mParent->app->changeStatusbar(tmpport->origin());
						if (tmpport == portsInView.at(currentRow)->p)
							return;
						
						if (currentRow >= 0) /// if last port is not finsihed yet mark it as waiting
							if (!portsInView.at(currentRow)->finished)
							{
								lOverview->setText(currentRow,1, i18n("waiting..."));
								lOverview->setPixmap(currentRow, 1, mParent->IconLoader.loadIcon((QString)"kalarm", KIcon::Small));
							}
								
						currentRow = -1;
							
						for (int i = 0; i < portsInView.count(); ++i) /// find the row
							if (portsInView.at(i)->p == tmpport)
								currentRow = i;
								
						if (currentRow == -1) /// port wasnt in the list, has to be added
						{
							lOverview->insertRows(lOverview->numRows());/// insert one row at the end
							lOverview->setRowStretchable(lOverview->numRows()-1, false);
							lOverview->verticalHeader()->setLabel(lOverview->numRows()-1, tmpport->avSource()->name());
							if (Buffer.contains("Verifying install for"))
								lOverview->setText(lOverview->numRows()-1,0 , i18n("install(src)"));
							else
								lOverview->setText(lOverview->numRows()-1,0 , i18n("install"));
							lOverview->setPixmap(lOverview->numRows()-1,0 , mParent->IconLoader.loadIcon((QString)"forward", KIcon::Small));  
							lOverview->setRowReadOnly(lOverview->numRows()-1, true);
									
							/// save some info in this stringlist for later reference
							_portInView *entry = new _portInView;
							entry->p = tmpport;
							entry->bin = 0;
							entry->line = lOverview->numRows()-1;
							entry->finished = false;
							entry->errors = "";
							portsInView.append(entry);
									
							lOverview->adjustColumn(0);
									
							currentRow = lOverview->numRows() -1;
						}
//						mParent->app->changeStatusbar(tmpport->origin() + "  " + QString::number(currentRow));
						
					} else
//						KMessageBox::information(this, "Buffer:"+Buffer+"\nsplit:"+list.join("~") +"\n elemt 1 vom grep<" + list.grep("/").first().section('/',3,4) + ">");
						KMessageBox::information(this, "The Install Process refers to a Port, that is not available in your Tree. "
										"The Port could be broken, or your Packages might be out of sync with your "
										"Portstree. Updating the Portstree should solve the problem.\n"
										"If you encounter problems with this Port after install, reinstall it after "
										"having updated the Portstree.");
					
					
	//			} else if (Buffer.startsWith("--->  Found a Package"))	
				} else if (Buffer.startsWith("--->  Fetching the Package")) /// fetching (bin)
				{
					/// make sure this is actually displayed as binary install/upgrade
					if (!lOverview->text(currentRow, 0).contains("(bin)"))
					{
						portsInView.at(currentRow)->bin = 1;
						lOverview->setText(currentRow, 0, lOverview->text(currentRow, 0).replace("(src)", "(bin)"));
					}
					
					lOverview->setText(currentRow, 1, i18n("fetching"));
				} else if (Buffer.startsWith("=> Attempting to fetch") || Buffer.contains("--->  Invoking a command: /usr/bin/fetch")) /// fetching (src)
				{
					//KMessageBox::information(this, "attempting to fetch");
					lOverview->setText(currentRow, 1, i18n("fetching"));
				} else if (Buffer.contains("===>  Extracting"))
				{
					lOverview->setText(currentRow, 1, i18n("extracting"));
				} else if (Buffer.contains("Checksum OK"))
				{
					lOverview->setText(currentRow, 1, i18n("Checksum Ok"));
				} else if (Buffer.contains("% of")) //&& (lOverview->text(currentRow,1).startsWith(i18n("Fetching"))))
				{
					Buffer.replace('_',' ');
					QStringList list = QStringList::split(' ', Buffer);
					if (Buffer.contains("100%")) /// at 100% there is no space between file and perecnt
						lOverview->setText(currentRow, 1, i18n("fetching")+"(100%)");
					else
						lOverview->setText(currentRow, 1, i18n("fetching")+"("+list.grep("%").first()+")");
				} else if (Buffer.contains("Installing") && Buffer.contains("from a port")) /// installing (whatever that means) (src
				{
					/// make sure this is actually displayed as source install/upgrade
					if (!lOverview->text(currentRow, 0).contains("(src)"))
					{
						portsInView.at(currentRow)->bin = 0;
						lOverview->setText(currentRow, 0, lOverview->text(currentRow, 0).replace("(bin)", "(src)"));
					}
					/// change the version in the table to that which portupgrade really is about to install (should be the same)
					/// but might differ, if port is new no version is there yet
//					if (lOverview->text(0, currentRow).contains(" of ") || lOverview->text(0, currentRow).contains(" to "))
//						lOverview->setText(currentRow, 0, lOverview->text(0, currentRow).section(' ',0, -2)); /// remove old version string
						
					///and append new one
//					lOverview->setText(currentRow, 0, lOverview->text(0, currentRow).append(" " + Buffer.section('\'',1,1).section('-',-1,-1)));
					
					lOverview->setText(currentRow, 1, i18n("installing"));
				} else if (Buffer.contains("Installing") && Buffer.contains("package")) /// installing (whatever that means) I
				{
					/// make sure this is actually displayed as binary install/upgrade
					if (!lOverview->text(currentRow, 0).endsWith("(bin)"))
					{
						portsInView.at(currentRow)->bin = 1;
						lOverview->setText(currentRow, 0, lOverview->text(currentRow, 0).replace("(src)", "(bin)"));
					}
					/// change the version in the table to that which portupgrade really is about to install (should be the same)
					/// but might differ, if port is new no version is there yet
//					if (lOverview->text(0, currentRow).contains(" of ") || lOverview->text(0, currentRow).contains(" to "))
//						lOverview->setText(currentRow, 0, lOverview->text(0, currentRow).section(' ',0, -2)); /// remove old version string
						
					///and append new one
//					lOverview->setText(currentRow, 0, lOverview->text(0, currentRow).append(" " + Buffer.section('\'',1,1).section('-',-1,-1)));
					
					lOverview->setText(currentRow, 1, i18n("installing"));
				} else if (	( (Buffer.contains("ended at:") && !Buffer.contains("Session")) ||
					  	  (Buffer.contains("Registering installation for")) ) 
						 &&  (Buffer.contains(portsInView.at(currentRow)->p->origin()) ||
							(Buffer.contains(lOverview->verticalHeader()->label(currentRow))) ) )/// an install is finished 
				{
					lOverview->setText(currentRow, 1, i18n("finished"));
					portsInView.at(currentRow)->finished=true;
					lOverview->setPixmap(currentRow, 1, mParent->IconLoader.loadIcon((QString)"bottom", KIcon::Small));
					bool allFinished = true;				
					for (int i = 0; i < portsInView.count(); ++i)
					{
						if (!portsInView.at(i)->finished)
							allFinished = false;
					
					}
					if (allFinished)
						tExtra->setText(i18n("<p align=\"center\">Registering Changes...</p>"));
					
				} else if (Buffer.contains("Listing the results")) /// results are shown
				{
					awaitingResults = true;
				} else if (awaitingResults)
				{		
					if (Buffer.startsWith("--->  Packages processed:"))
						awaitingResults = false;
					else
					{
						QBsdPort *tmpport;
						QString namepart = Buffer.section('\t',0,0,QString::SectionSkipEmpty).section(' ',1,1);
						
						if (namepart.contains('/')) /// origin
							tmpport = QBsdPort::global->portWithOrigin(namepart);
						else if (namepart.contains('@')) /// name with @ means latest package
							tmpport = QBsdPort::global->portWithName(namepart.remove('@'),0, "");
						else
							tmpport = QBsdPort::global->portWithName(namepart.section('-',0,-2), 0, namepart.section('-',-1,-1));
							
						if (tmpport)
						{
							int row = -1;
							for (int i=0; i< portsInView.count(); ++i)
								if (portsInView.at(i)->p == tmpport)
									row = i;
									
							if (row == -1)
								return;
							
							QString	result	= Buffer.section('\t',0,0,QString::SectionSkipEmpty).section(' ',0,0);
							if (result=="+")
							{
								portsInView.at(row)->errors	= "";
								portsInView.at(row)->finished	= true;
							} else if (result=="!")
							{
								portsInView.at(row)->errors	+= Buffer.section('\t',-1,-1).remove('(').remove(')') + "\n";
								portsInView.at(row)->finished	= true;
							} else if (result=="*")
							{
								portsInView.at(row)->errors	= "skipped\n";
								portsInView.at(row)->finished	= true;
							} else if (result=="-")
							{
								portsInView.at(row)->errors	= "ignored\n";
								portsInView.at(row)->finished	= true;
							}
						}
						
					}
				} else if (Buffer.contains("Session ended")) /// installing (whatever that means)
				{
					// ended, clear listview and display results
					setTitle(page(2), i18n("Step 3 - Install     Results"));
					
					lOverview->insertColumns(lOverview->numCols());
					lOverview->horizontalHeader()->setLabel(lOverview->numCols()-2, i18n("Result"));
					lOverview->horizontalHeader()->setLabel(lOverview->numCols()-1, i18n("Message"));
					
					for (int i=0; i< portsInView.count(); ++i)
					{
						portsInView.at(i)->p->findInstalled(); /// recheck  installed version info
							
						if (portsInView.at(i)->finished)
						{
							if (portsInView.at(i)->errors == "")
							{
//								lOverview->setText(i, 1, i18n("installed"));
//								lOverview->setPixmap(i, 1, mParent->IconLoader.loadIcon((QString)"ok", KIcon::Small));
								
									
								if (portsInView.at(i)->p->installed())
								{
									lOverview->setText(i, 1, i18n("installed"));
									lOverview->setPixmap(i, 1, mParent->IconLoader.loadIcon((QString)"ok", KIcon::Small));
										
									if (portsInView.at(i)->p->installed()->message() != "")
									{
										QPortLink *item = new QPortLink(this,((QString)"pMessage"+portsInView.at(i)->p->origin()), portsInView.at(i)->p);
										item->setText(i18n("view"));
										item->setPaletteBackgroundColor(backgroundColor);
										lOverview->clearCellWidget(i, 2); /// delete old item
										lOverview->setCellWidget(i, 2, item); /// set new one
										connect(item, SIGNAL(clickedPort(QBsdPort*)), this, SLOT(messagePushed(QBsdPort*)));
									} else
										lOverview->setText(i, 2, i18n("none"));
								}
							} else
							{
//								lOverview->setPixmap(i, 1, mParent->IconLoader.loadIcon((QString)"error", KIcon::Small));
								
								QPortLink *item = new QPortLink(this,((QString)"pFailed"+portsInView.at(i)->p->origin()), portsInView.at(i)->p);
								item->setText(i18n("failed"));
								item->setPaletteBackgroundColor(backgroundColor);
								lOverview->clearCellWidget(i, 1); /// delete old item
								lOverview->setCellWidget(i, 1, item);
								connect(item, SIGNAL(clickedPort(QBsdPort*)), this, SLOT(failedPushed(QBsdPort*)));	
								
							}
									
							//threads zum neueinlesen der instversoin starten und connecten
						} else
							lOverview->setText(i, 1, "?");
										
						updateLWItemsInParent(portsInView.at(i)->p);
					}
											
					finishButton()->setEnabled(true);
					cancelButton()->setEnabled(false);
					lOverview->adjustColumn(2);
					tExtra->setText("");
										
					int instnum = 0;				
					for (QBsdPort *po = QBsdPort::global->array.first(); po; po = QBsdPort::global->array.next())
						if (po->installed())
							++instnum;

					mParent->MainTab->setTabLabel(mParent->MainTab->page(1), i18n("Installed ") + "[" 
										+ QString::number(instnum) + "]" );
					
				} else if (Buffer.contains("interrupted by User.")) ///
				{
					wasInterupted = true;
					tExtra->setText(i18n("<font color=\"#BB0000\"><p align=\"center\">The Process was interrupted, resuls may be incorrect.</font></p>"));
				} else if (Buffer.contains("pkgdb -F")) /// the pkgdb is broken
				{
					if (KMessageBox::warningYesNo(this, i18n("Your Package Database is broken. It needs to be fixed"
						" manually."
						"\nStart a a Fix-it Shell?"))
						 == KMessageBox::Yes)
					{
						KRun::runCommand("konsole -e pkgdb -F");						
					}
				}
				
				} break;
		}
	} else /// uninstall
	{
		switch (QBsdPort::global->platform.pkgsystem)
		{
			case -1: KMessageBox::information(this, i18n("No simplified Progress information is calculated when Portupgrade is not used."
									"Use Portupgrade or klick on \"Konsole\" to view unfiltered output."));
				break;
			case 0:{
				if (Buffer.contains("--->  Deinstalling"))
				{
					QBsdPort* tmpport = QBsdPort::global->portWithName(Buffer.section('\'', 1, 1).section('-',0,-2), 3, Buffer.section('\'', 1, 1).section('-',-1,-1));
					
					if (tmpport)
					{
					
						if (currentRow >= 0) /// mark last ports as finished
							if (lOverview->text(currentRow, 1) == i18n("removing"))
							{
								lOverview->setText(currentRow,1, i18n("finished"));
								portsInView.at(currentRow)->finished = true;
								lOverview->setPixmap(currentRow, 1, mParent->IconLoader.loadIcon((QString)"bottom", KIcon::Small));
								bool allFinished = true;				
								for (int i = 0; i < portsInView.count(); ++i)
								{
									if (!portsInView.at(i)->finished)
										allFinished = false;
								
								}
								if (allFinished)
									tExtra->setText(i18n("<p align=\"center\">Registering Changes...</p>"));
							}
						currentRow = -1;
							
						for (int i = 0; i < portsInView.count(); ++i) /// find the row
							if (portsInView.at(i)->p == tmpport)
								currentRow = i;
								
						if (currentRow != -1)
							lOverview->setText(currentRow, 1, i18n("removing"));
						else
						{
							lOverview->insertRows(lOverview->numRows());/// insert one row at the end
							lOverview->setRowStretchable(lOverview->numRows()-1, false);
							lOverview->verticalHeader()->setLabel(lOverview->numRows()-1, tmpport->installed()->name());

							lOverview->setPixmap(lOverview->numRows()-1,0 , mParent->IconLoader.loadIcon((QString)"trashcan_empty", KIcon::Small));  
							lOverview->setRowReadOnly(lOverview->numRows()-1, true);
									
							/// save some info in this stringlist for later reference
							_portInView *entry = new _portInView;
							entry->p = tmpport;
							entry->bin = 0;
							entry->line = lOverview->numRows()-1;
							entry->finished = false;
							entry->errors = "";
							portsInView.append(entry);
									
							lOverview->adjustColumn(0);
									
							currentRow = lOverview->numRows() -1;
						} 
					}
				} else if (Buffer.contains("interrupted by User.")) ///
				{
					wasInterupted = true;
					tExtra->setText(i18n("<font color=\"#BB0000\"><p align=\"center\">The Process was interrupted, resuls may be incorrect.</font></p>"));
				} else if (Buffer.contains("Listing the results")) /// results are shown
				{
					awaitingResults = true;
				}  else if (Buffer.startsWith("--->  Packages processed:")) /// overview finished
				{
					// ended, clear listview and display results
					setTitle(page(2), i18n("Step 3 - Uninstall     Results"));
					
//					lOverview->insertColumns(lOverview->numCols());
					lOverview->horizontalHeader()->setLabel(lOverview->numCols()-1, i18n("Result"));
//					lOverview->horizontalHeader()->setLabel(lOverview->numCols()-1, i18n("Message"));
					
					for (int i=0; i< portsInView.count(); ++i)
					{
						portsInView.at(i)->p->findInstalled(); /// recheck  installed version info
							
						if (portsInView.at(i)->finished)
						{
							if (portsInView.at(i)->errors == "")
							{
//								lOverview->setText(i, 1, i18n("installed"));
//								lOverview->setPixmap(i, 1, mParent->IconLoader.loadIcon((QString)"ok", KIcon::Small));
								
									
								if (!portsInView.at(i)->p->installed())
								{
									lOverview->setText(i, 1, i18n("removed"));
									lOverview->setPixmap(i, 1, mParent->IconLoader.loadIcon((QString)"ok", KIcon::Small));
	
								}
							} else
							{
//								lOverview->setPixmap(i, 1, mParent->IconLoader.loadIcon((QString)"error", KIcon::Small));
								
								QPortLink *item = new QPortLink(this,((QString)"pFailed"+portsInView.at(i)->p->origin()), portsInView.at(i)->p);
								item->setText(i18n("failed"));
								item->setPaletteBackgroundColor(backgroundColor);
								lOverview->setCellWidget(i, 1, item);
								connect(item, SIGNAL(clickedPort(QBsdPort*)), this, SLOT(failedPushed(QBsdPort*)));	
								
							}
									
							//threads zum neueinlesen der instversoin starten und connecten
						} else
							lOverview->setText(i, 1, "?");

						updateLWItemsInParent(portsInView.at(i)->p);
						
					}
									
					int instnum = 0;				
					for (QBsdPort *po = QBsdPort::global->array.first(); po; po = QBsdPort::global->array.next())
						if (po->installed())
							++instnum;

					mParent->MainTab->setTabLabel(mParent->MainTab->page(1), i18n("Installed ") + "[" 
										+ QString::number(instnum) + "]" );
					finishButton()->setEnabled(true);
					cancelButton()->setEnabled(false);
					tExtra->setText("");

					
				} else if (awaitingResults) /// there is only one overview when deinstalling
				{	
					QBsdPort *tmpport;
					QString namepart = Buffer.section('\t',0,0,QString::SectionSkipEmpty).section(' ',1,1);
					
					tmpport = QBsdPort::global->portWithName(namepart.section('-',0,-2), 3, namepart.section('-',-1,-1));
						
					if (tmpport)
					{
						int row = -1;
						for (int i=0; i< portsInView.count(); ++i)
							if (portsInView.at(i)->p == tmpport)
								row = i;
								
						if (row == -1)
							return;
						
						QString	result	= Buffer.section('\t',0,0,QString::SectionSkipEmpty).section(' ',0,0);
						if (result=="+")
						{
							portsInView.at(row)->errors	= "";
							portsInView.at(row)->finished	= true;
						} else if (result=="!")
						{
							portsInView.at(row)->errors	+= Buffer.section('\t',-1,-1).remove('(').remove(')') + "\n";
							portsInView.at(row)->finished	= true;
						} else if (result=="*")
						{
							portsInView.at(row)->errors	= "skipped\n";
							portsInView.at(row)->finished	= true;
						} else if (result=="-")
						{
							portsInView.at(row)->errors	= "ignored\n";
							portsInView.at(row)->finished	= true;
						}
					}
					
					
				} else if (Buffer.contains("pkgdb -F")) /// the pkgdb is broken
				{
					if (KMessageBox::warningYesNo(this, i18n("Your Package Database is broken. It needs to be fixed"
						" manually."
						"\nStart a a Fix-it Shell?"))
						 == KMessageBox::Yes)
					{
						KRun::runCommand("konsole -e pkgdb -F");						
					}
				}
				} break;
										
		}
	}
							
	/// determine if the exta info line (bottom between the two buttons)
	/// is to hold a value
	if (tExtra->text() == i18n("<p align=\"center\">Preparing...</p>")) ///only do this every 4th line to save resources
	{
		bool nothingYet = true;				
		for (int i = 0; i < lOverview->numRows(); ++i)
		{
			if (lOverview->text(i, 1) != "")
				nothingYet = false;
		}
		if (!nothingYet)
			tExtra->setText("");
	}
		
	lOverview->adjustColumn(0);
	lOverview->adjustColumn(1);
}
						
void	InstallWizard::updateLWItemsInParent(QBsdPort *p)
{

//	qDebug("processing: " +  p->origin());
	QListViewItem *tmpitem, *tmpparent;
	bool end = false;
	
	do /// main list view
	{ 
		qDebug("in zeile " + QString::number(__LINE__));
		tmpparent = mParent->ListView->findItem(p->origin().section('/',0,0), 0); /// find category
		if (!tmpparent)
		{
//			qDebug("error: information in parent cant be updated. category not found");
			return;
		}
	
//		qDebug("parent = " + tmpparent->text(0));
	
	} while (tmpparent->parent() != 0); ///make sure its a toplevel widget
		
	for (tmpitem = tmpparent->firstChild(); (!end); )
	{
		
		if (tmpitem == 0)
			end = true;
		else if (tmpitem->text(0) == p->origin().section('/',1,1))
		
			end = true;
		else
		{
//			qDebug("parent = " + tmpparent->text(0) + " und child = " + tmpitem->text(0));
			tmpitem = tmpitem->nextSibling();
		}
	}
	if (!tmpitem)
	{
//		qDebug("holla");
		return;
	} else if (p->installed())
	{
//		qDebug("updating... " + tmpitem->text(1) + " to  " + p->installed()->version());
		tmpitem->setText(1, p->installed()->version());
	} else
	{
//		qDebug("clearing...");
		tmpitem->setText(1, "");
	}
				
	mParent->ListView->triggerUpdate();
		
	tmpitem = 0; tmpparent = 0; end = 0;
			
	do /// installed listview
	{ 		
		tmpparent = mParent->ListViewInst->findItem(p->origin().section('/',0,0), 0); /// find category
		if (!tmpparent)
		{
//			qDebug("error: information in parent cant be updated. category not found");
			return;
		}
//		qDebug("parent = " + tmpparent->text(0));
	
	} while (tmpparent->parent() != 0); ///make sure its a toplevel widget
		
	for (tmpitem = tmpparent->firstChild(); (!end); )
	{		
		if (tmpitem == 0)
			end = true;
		else if (tmpitem->text(0) == p->origin().section('/',1,1))
		
			end = true;
		else
		{
//			qDebug("parent = " + tmpparent->text(0) + " und child = " + tmpitem->text(0));
			tmpitem = tmpitem->nextSibling();
		}

	}	
	if (!tmpitem)
	{
//		qDebug("inserting...");
		mParent->insertPortInView(mParent->ListViewInst, p);
	} else if (p->installed())
	{
//		qDebug("changing...");
		tmpitem->setText(1, p->installed()->version());
	} else
	{
//		qDebug("removing...");
		tmpparent->takeItem(tmpitem);
		delete tmpitem;
	}
	mParent->ListViewInst->triggerUpdate();
}
						
						
void InstallWizard::updateIcon()
{
	/// function switches between 1arrow and 2arrow for currentRows icon to show that something is happening
						
	if (currentRow < 0)
		return;

	if ( (portsInView.at(currentRow)->finished) || (lOverview->text(currentRow, 1) == i18n("waiting...")) || (lOverview->text(currentRow, 1) == ""))
		return;

	
	if (show2arrow)
	{
		lOverview->setPixmap(currentRow, 1, mParent->IconLoader.loadIcon((QString)"2rightarrow", KIcon::Small));
		show2arrow = false;
	} else
	{
		lOverview->setPixmap(currentRow, 1, mParent->IconLoader.loadIcon((QString)"1rightarrow", KIcon::Small));
		show2arrow = true;
	}

	lOverview->adjustColumn(1);

}
						
void InstallWizard::stdErrReceived(QString Buffer)
{
	stdOutReceived("<font color=\"#ff0000\">"+ Buffer+"</font>");
//	stdOutReceived(Buffer + "  <-> error out");
}

void InstallWizard::installProcFinished()
{

	timer->stop();
	if (logFile)		
		if (logFile->isOpen())
		{
			logFile->close();
			delete logFile;
		}
		
	bCommit->setText(i18n("retry"));
	bCommit->setIconSet(mParent->IconLoader.loadIcon((QString)"run", KIcon::Small));
	backButton()->setEnabled(true);	
	bCommit->setEnabled(true);
	if (!title(page(2)).endsWith(i18n("Results")))
	{
		
		if (aim != -1 )
		{
			setTitle(page(2),i18n("Step 3 - Install     <font color=\"#ff0000\">Portupgrade-Error</font>"));
			
			KMessageBox::information(this, i18n("The Install Process terminated abnormally. "
						"Therefor results cannot by shown. Pressing \"Konsole\" might give more information."),
						QString(), "dontWarnNoResults");
		} else
		{
			setTitle(page(2),i18n("Step 3 - Uninstall     <font color=\"#ff0000\">Error</font>"));
			
			KMessageBox::information(this, i18n("The Uninstall Process terminated abnormally. "
						"Therefor results cannot by shown. Pressing \"Konsole\" might give more information."),
						QString(), "dontWarnNoResults");					
		}
	}

}

void InstallWizard::installCmdReceived(QString cmd)
{

	lCmd->setText(cmd);

}
					
void InstallWizard::failedPushed(QBsdPort* Port)
{
	int row = -1;
	for (int i=0; i< portsInView.count(); ++i)
		if (portsInView.at(i)->p == Port)
			row = i;
	if (row==-1)
		return;
	
	KMessageBox::information(this, portsInView.at(row)->errors);

}


void InstallWizard::messagePushed(QBsdPort* Port)
{
	KMessageBox::information(this, Port->installed()->message());

}
/*
void InstallWizard::saveLog()
{
	QFile outfile(KFileDialog::getSaveFileName(":export", QString(), this, i18n("KPorts - Save As...")));
	
	if (!outfile.open(IO_WriteOnly))
	{
		KMessageBox::error(this, i18n("Could not open file for writing."));
		return;
	}
	
	outfile.writeBlock(tOutput->text(), tOutput->text().length());
	outfile.close();
			
}*/

InstallWizard::~InstallWizard()
{
	if (!backButton()->isEnabled())
	{
		if (geteuid()!=0)
		{
			KMessageBox::error(this, i18n("The Process could not be aborted, because you do not have sufficient "
							"previliges. You should start KPorts as root to avoid this."));

		} else
		{
			if (aim>=0)
				QBsdPort::global->stopProc("installProc");
			else
				QBsdPort::global->stopProc("uninstallProc");
		}

	}
}
