/***************************************************************************
 *   Copyright (C) 2005 by Hannes Hauswedell                               *
 *   soul.rebel@web.de                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kportspopups.h"

#include "kports.h"

PortWarningDlg::PortWarningDlg( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "PortWarningDlg" );
    Form1Layout = new QGridLayout( this, 1, 1, 11, 6, "Form1Layout"); 

    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, 0, 0, sizePolicy().hasHeightForWidth() ) );

    secGroup = new QButtonGroup( this, "secGroup" );
    secGroup->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)1, 0, 0, secGroup->sizePolicy().hasHeightForWidth() ) );
    secGroup->setColumnLayout(0, Qt::Vertical );
    secGroup->layout()->setSpacing( 6 );
    secGroup->layout()->setMargin( 11 );
    secGroupLayout = new QGridLayout( secGroup->layout() );
    secGroupLayout->setAlignment( Qt::AlignTop );
    secGroup->layout()->setAutoAdd(true);

    Form1Layout->addWidget( secGroup, 1, 0 );

    flagGroup = new QButtonGroup( this, "flagGroup" );
    flagGroup->setColumnLayout(0, Qt::Vertical );
    flagGroup->layout()->setSpacing( 6 );
    flagGroup->layout()->setMargin( 11 );
    flagGroupLayout = new QGridLayout( flagGroup->layout() );
    flagGroupLayout->setAlignment( Qt::AlignTop );

    cBroken = new QCheckBox( flagGroup, "cBroken" );
    cBroken->setEnabled( FALSE );
    cBroken->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, 0, 0, cBroken->sizePolicy().hasHeightForWidth() ) );
    cBroken->setPaletteForegroundColor( QColor( 255, 0, 0 ) );
    cBroken->setChecked( TRUE );
    

    flagGroupLayout->addWidget( cBroken, 0, 0 );

    cIgnore = new QCheckBox( flagGroup, "cIgnore" );
    cIgnore->setEnabled( FALSE );
    cIgnore->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, 0, 0, cIgnore->sizePolicy().hasHeightForWidth() ) );
    cIgnore->setPaletteForegroundColor( QColor( 255, 0, 0 ) );
    cIgnore->setChecked( TRUE );

    flagGroupLayout->addWidget( cIgnore, 1, 0 );

    cForbidden = new QCheckBox( flagGroup, "cForbidden" );
    cForbidden->setEnabled( FALSE );
    cForbidden->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, 0, 0, cForbidden->sizePolicy().hasHeightForWidth() ) );
    cForbidden->setPaletteForegroundColor( QColor( 255, 0, 0 ) );
    cForbidden->setChecked( TRUE );

    flagGroupLayout->addWidget( cForbidden, 2, 0 );

    cRestricted = new QCheckBox( flagGroup, "cRestricted" );
    cRestricted->setEnabled( FALSE );
    cRestricted->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, 0, 0, cRestricted->sizePolicy().hasHeightForWidth() ) );
    cRestricted->setPaletteForegroundColor( QColor( 255, 0, 0 ) );
    cRestricted->setChecked( TRUE );

    flagGroupLayout->addWidget( cRestricted, 3, 0 );

    cDeprecated = new QCheckBox( flagGroup, "cDeprecated" );
    cDeprecated->setEnabled( FALSE );
    cDeprecated->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, 0, 0, cDeprecated->sizePolicy().hasHeightForWidth() ) );
    cDeprecated->setPaletteForegroundColor( QColor( 255, 0, 0 ) );
    cDeprecated->setChecked( TRUE );

    flagGroupLayout->addWidget( cDeprecated, 4, 0 );

    Form1Layout->addWidget( flagGroup, 2, 0 );

    updGroup = new QButtonGroup( this, "updGroup" );
    updGroup->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, updGroup->sizePolicy().hasHeightForWidth() ) );
    updGroup->setFrameShape( QButtonGroup::GroupBoxPanel );
    updGroup->setFrameShadow( QButtonGroup::Sunken );
    updGroup->setColumnLayout(0, Qt::Vertical );
    updGroup->layout()->setSpacing( 6 );
    updGroup->layout()->setMargin( 11 );
    updGroupLayout = new QGridLayout( updGroup->layout() );
    updGroupLayout->setAlignment( Qt::AlignTop );

    eUpdating = new QTextEdit( updGroup, "eUpdating" );
    eUpdating->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, 0, 0, eUpdating->sizePolicy().hasHeightForWidth() ) );
    eUpdating->setReadOnly( TRUE );

    updGroupLayout->addWidget( eUpdating, 0, 0 );

    Form1Layout->addWidget( updGroup, 0, 0 );

    buttonGroup = new QButtonGroup( this, "buttonGroup" );
    buttonGroup->setFlat( TRUE );
    buttonGroup->setColumnLayout(0, Qt::Vertical );
    buttonGroup->layout()->setSpacing( 6 );
    buttonGroup->layout()->setMargin( 11 );
    buttonGroupLayout = new QGridLayout( buttonGroup->layout() );
    buttonGroupLayout->setAlignment( Qt::AlignTop );

    okButton = new QPushButton( buttonGroup, "okButton" );
    okButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, 0, 0, okButton->sizePolicy().hasHeightForWidth() ) );

    buttonGroupLayout->addWidget( okButton, 0, 1 );
    spacer1 = new QSpacerItem( 91, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
    buttonGroupLayout->addItem( spacer1, 0, 0 );
    spacer2 = new QSpacerItem( 101, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
    buttonGroupLayout->addItem( spacer2, 0, 2 );

    Form1Layout->addWidget( buttonGroup, 3, 0 );

    resize( QSize(441, 492).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );
	
    connect(okButton, SIGNAL(clicked()), this, SLOT(close()));
			    
//    setCaption( i18n( "Form1" ) );
    secGroup->setTitle( i18n( "Security Warnings" ) );
    flagGroup->setTitle( i18n( "marked as" ) );
    QToolTip::add( flagGroup, i18n( "Some of these flags are critical and will prevent build/install." ) );
  /*  cBroken->setText( i18n( "\"BRO&KEN\"" ) );
    cBroken->setAccel( QKeySequence( i18n( "Alt+K" ) ) );
    cIgnore->setText( i18n( "\"&IGNORE\"" ) );
    cIgnore->setAccel( QKeySequence( i18n( "Alt+I" ) ) );
    cForbidden->setText( i18n( "\"FORBIDDE&N\"" ) );
    cForbidden->setAccel( QKeySequence( i18n( "Alt+N" ) ) );
    cRestricted->setText( i18n( "\"&RESTRICTED\"" ) );
    cRestricted->setAccel( QKeySequence( i18n( "Alt+R" ) ) );
    cDeprecated->setText( i18n( "\"DEPRE&CATED\"" ) );
    cDeprecated->setAccel( QKeySequence( i18n( "Alt+C" ) ) );*/
    updGroup->setTitle( i18n( "\"Updating-File\" warnings" ) );
    QToolTip::add( eUpdating, i18n( "see \"What's this?\" for explanation" ) );
    QWhatsThis::add( eUpdating, i18n( "The <b>Updating file</b> located in your ports directory, warns about changes in certain ports that can prevent a regular upgrade. \n"
"You should <font color=\"#ff0000\" face=\"Voreinstellung\">read the Warnings carefully</font>  and see whether they apply to thechanges you wish perform on the port." ) );
    buttonGroup->setTitle( QString::null );
    okButton->setText( i18n( "Ok" ) );
}
			    
PortOptionsDlg::PortOptionsDlg( QWidget* parent, const char* name)
    : QDialog( parent, name, 0,0 )
{
    	if ( !name )
		setName( "PortOptionsDlg" );			    
	
	Form1Layout = new QGridLayout( this, 1, 1, 11, 6, "Form1Layout"); 
	
	setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, 0, 0, sizePolicy().hasHeightForWidth() ) );
	
	optionsGroup = new QButtonGroup( this, "optionsGroup" );
	optionsGroup->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)1, 0, 0, optionsGroup->sizePolicy().hasHeightForWidth() ) );
	optionsGroup->setColumnLayout(0, Qt::Vertical );
	optionsGroup->layout()->setSpacing( 6 );
	optionsGroup->layout()->setMargin( 11 );
	optionsGroupLayout = new QGridLayout( optionsGroup->layout() );
	optionsGroupLayout->setAlignment( Qt::AlignTop );
	optionsGroup->layout()->setAutoAdd(true);
	optionsGroup->setTitle( i18n( "Build Options" ) );
				
	Form1Layout->addWidget( optionsGroup, 1, 0 );
	
	
	buttonGroup = new QButtonGroup( this, "buttonGroup" );
	buttonGroup->setFlat( TRUE );
	buttonGroup->setColumnLayout(0, Qt::Vertical );
	buttonGroup->layout()->setSpacing( 6 );
	buttonGroup->layout()->setMargin( 11 );
	buttonGroupLayout = new QGridLayout( buttonGroup->layout() );
	buttonGroupLayout->setAlignment( Qt::AlignTop );
	
	okButton = new QPushButton( buttonGroup, "okButton" );
	okButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, 0, 0, okButton->sizePolicy().hasHeightForWidth() ) );
	
	buttonGroupLayout->addWidget( okButton, 0, 1 );
	spacer1 = new QSpacerItem( 91, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	buttonGroupLayout->addItem( spacer1, 0, 0 );
	spacer2 = new QSpacerItem( 101, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	buttonGroupLayout->addItem( spacer2, 0, 2 );
	
	Form1Layout->addWidget( buttonGroup, 3, 0 );
	
	resize( QSize(441, 492).expandedTo(minimumSizeHint()) );
	clearWState( WState_Polished );
				
	okButton->setText( i18n( "Ok" ) );
	connect(okButton, SIGNAL(clicked()), this, SLOT(close()));
	
	
}