/***************************************************************************
 *   Copyright (C) 2005, 2006 by Hannes Hauswedell                         *
 *   hannes.hauswedell@gmail.com                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KPORTSVIEW_H
#define KPORTSVIEW_H

#include <kactioncollection.h>
#include <klistview.h>
#include <klistviewsearchline.h>
#include <kconfigskeleton.h>
#include <kpushbutton.h>
#include <kiconloader.h>
#include <ktextedit.h>
#include <kconfig.h>
#include <kurllabel.h>
#include <krun.h>
#include <kmessagebox.h>
#include <kcursor.h>
 
#include <qlabel.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qgroupbox.h>
#include <qthread.h>
#include <qlistbox.h>
#include <qtimer.h>
#include <qfile.h>
#include <qmutex.h>
#include <qevent.h>
#include <qapplication.h>
//#include <qwidgetstack.h>
#include <qcolor.h>
#include <qobjectlist.h>
#include <qtoolbutton.h>
#include <qtabwidget.h>
#include <qsplashscreen.h>
#include <qmessagebox.h>

#include <pwd.h>

/*
#include <stdlib.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>
*/
#include "bsdport.h"
#include "pref.h"
#include "installwizard.h"
#include "kports.h"
#include "kportsviewbase.h"
#include "updateavsrc.h"

class KPorts;
class KPortsPrefConfig;
class KPortsView;
class QBsdPort;


/**
The KPortsView widget Implementation

@author Hannes Hauswedell
*/

class KPortsView : public KPortsViewBase
{

Q_OBJECT
private:
	QBsdPort	*m_currentPort;
	int		m_currentPkg;
public:
	
	bool		isStarting;	/// see constructor and populateView() for info on use

	KListViewItem 	*currentItem;
	QBsdPort	*currentPort() { return m_currentPort; }
	void		setCurrentPort(QBsdPort*);
	
	///NOTE: the following does NOT return *QBsdPortPkg, instead it return 0 for QBsdPortInstPkg, 1 for avsrc..	
	int		currentPkg() { return m_currentPkg; }
	void		setCurrentPkg(int newValue) { m_currentPkg = newValue; }


	KPorts		*app;
	FILE		*installPipe;
	KListView	*currentListView();

	KIconLoader 	IconLoader;
	
	QMutex		fileViewMutex;


//	int GetFreeThread();
//	int PopulateList();

	void	clearTab();
	void	clearPortInfo();
	void	clearPkgInfo();
	void	updateTab();
	void	showInfo();
	
	/// the following two work differently but are used only for inserting ports into listviews
	void 	insertPortInView(QListView *view, QBsdPort* Port);
	void	pathToListViewItems(QListView *view, QString path, bool leadingSlash, bool allowDuplicate);
	
	KPortsView(QWidget *parent = 0, const char *name = 0);
	~KPortsView();
   
	

	/* OPTIONS */
	static bool g_bHideMainOnInstall; //hide mainwindow while in wizard
	static bool g_bListLoad;	  //load new and installed versions of port on display
	static bool g_bStateLoad;	  //check whether deps are installed in installwizards before installing
//	static bool g_bFilesTree;	  //whether files will be listed in treeview
	static bool sharedInfo;
	static QString g_sPrefix;	  //a global prefix to install to

	static KPortsPrefConfig	*Settings; //static pointer to settings, so they are accessible globally
	static KPortsView	*mainWnd;


	void	loadConfig();

//signals:
	
	
 //  	void sendEmail(const QString& name, const QString& email);
//	void openURL(const QString& url);
		
private:
//	void KPortsView::customEvent(QCustomEvent *event);
	//all ThreadedAction 1-15 have one corresponding event with type=1000 + action
	

	

public slots:
    virtual void populateView();
    virtual void NewItemSelected();
    virtual void InstallPressed();
    virtual void UninstallPressed();
    virtual void ItemExpanded(QListViewItem*);
    virtual void ItemCollapsed(QListViewItem*);
    virtual void CollapseAll();
    virtual void ExpandAll();
    virtual void MaintClicked();
    virtual void eSearchDescrPressed();
    virtual void URLClicked(const QString& url);
    void 	 mouseLeftURL();


	void	indexFileNotFound();
	void	vulnaDbOutdated();
	void	vulnaDbUpdatedSuccessfully();
	void	vulnaDbUpdatedUnsuccessfully(QString error);
	void	avBinProgress(int);
	void	pkgExportedSuccessfully();
	void	pkgExportedUnsuccessfully(QString error);

	/// these slots are called when the package information database is (re)built
	void	allAvSrcFound();
	void	allInstFound();	
	void	allAvBinFound();
	void	updFileRead();
	void	vulnasFound();

	/// the slots are necessary because certain information is not retrieved on start
	void	refreshInstSize();
	void	refreshInstFalseFiles();
	void	refreshVulna();
	void	refreshSrcDescr();
	void	refreshSrcFiles();
	void	refreshSrcOptions();
	void	refreshFlags();

	
	/// a search finished
	void	searchFinished(QPtrList <QBsdPort> result);
	
	/// menu / toolbar - action slots
	void	updateVulnaDb();
	void	updatePorts();
	void	upgradeAll();
	
	void	showUpdRest();
	void	showMoved();
	

    virtual void installViewFilterChanged(int);
    virtual void exportPkgClicked();
    virtual void avBinVersionSelected();
    virtual void avSrcVersionSelected();
    virtual void instVersionSelected();


protected:
    QListViewItem *q_Cat,*q_Port;


};




#endif
